# -*- coding: utf-8 -*-
"""Tests for Firefox cache files parser."""

import unittest

from plaso.formatters import firefox_cache  # pylint: disable=unused-import
from plaso.lib import errors
from plaso.lib import timelib
from plaso.parsers import firefox_cache

from tests import test_lib as shared_test_lib
from tests.parsers import test_lib


__author__ = 'Petter Bjelland (petter.bjelland@gmail.com)'


class FirefoxCacheParserTest(test_lib.ParserTestCase):
  """A unit test for the FirefoxCacheParser."""

  def _VerifyMajorMinor(self, events):
    """Verify that valid Firefox cache version is extracted."""
    for event_object in events:
      self.assertEqual(event_object.major, 1)
      self.assertEqual(event_object.minor, 19)

  @shared_test_lib.skipUnlessHasTestFile([u'firefox_cache', u'invalid_file'])
  def testParseCache_InvalidFile(self):
    """Verify that parser do not accept small, invalid files."""
    parser_object = firefox_cache.FirefoxCacheParser()
    path_segments = [u'firefox_cache', u'invalid_file']

    with self.assertRaises(errors.UnableToParseFile):
      self._ParseFile(path_segments, parser_object)

  @shared_test_lib.skipUnlessHasTestFile([
      u'firefox_cache', u'firefox28', u'_CACHE_001_'])
  def testParseCache_001(self):
    """Test Firefox 28 cache file _CACHE_001_ parsing."""
    parser_object = firefox_cache.FirefoxCacheParser()
    storage_writer = self._ParseFile(
        [u'firefox_cache', u'firefox28', u'_CACHE_001_'], parser_object)

    self.assertEqual(len(storage_writer.events), 1665)

    event_object = storage_writer.events[3]
    self.assertEqual(
        event_object.url, u'HTTP:http://start.ubuntu.com/12.04/sprite.png')

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2014-04-21 14:13:35')
    self.assertEqual(event_object.timestamp, expected_timestamp)

    expected_msg = (
        u'Fetched 2 time(s) '
        u'[HTTP/1.0 200 OK] GET '
        u'"HTTP:http://start.ubuntu.com/12.04/sprite.png"')
    expected_msg_short = (
        u'[HTTP/1.0 200 OK] GET '
        u'"HTTP:http://start.ubuntu.com/12.04/sprite.png"')

    self._TestGetMessageStrings(
        event_object, expected_msg, expected_msg_short)

    self._VerifyMajorMinor(storage_writer.events)

  @shared_test_lib.skipUnlessHasTestFile([
      u'firefox_cache', u'firefox28', u'_CACHE_002_'])
  def testParseCache_002(self):
    """Test Firefox 28 cache file _CACHE_002_ parsing."""
    parser_object = firefox_cache.FirefoxCacheParser()
    storage_writer = self._ParseFile(
        [u'firefox_cache', u'firefox28', u'_CACHE_002_'], parser_object)

    expected_url = (
        u'HTTP:http://www.google-analytics.com/__utm.gif?utmwv=5.5.0&utms='
        u'1&utmn=1106893631&utmhn=www.dagbladet.no&utmcs=windows-1252&ut'
        u'msr=1920x1080&utmvp=1430x669&utmsc=24-bit&utmul=en-us&utmje=0&'
        u'utmfl=-&utmdt=Dagbladet.no%20-%20forsiden&utmhid=460894302&utm'
        u'r=-&utmp=%2F&utmht=1398089458997&utmac=UA-3072159-1&utmcc=__ut'
        u'ma%3D68537988.718312608.1398089459.1398089459.1398089459.1%3B%'
        u'2B__utmz%3D68537988.1398089459.1.1.utmcsr%3D(direct)%7Cutmccn'
        u'%3D(direct)%7Cutmcmd%3D(none)%3B&aip=1&utmu=qBQ~')

    self.assertEqual(len(storage_writer.events), 141)

    event_object = storage_writer.events[5]
    self.assertEqual(event_object.url, expected_url)

    event_object = storage_writer.events[1]
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2014-04-21 14:10:58')
    self.assertEqual(event_object.timestamp, expected_timestamp)

    self._VerifyMajorMinor(storage_writer.events)

  @shared_test_lib.skipUnlessHasTestFile([
      u'firefox_cache', u'firefox28', u'_CACHE_003_'])
  def testParseCache_003(self):
    """Test Firefox 28 cache file _CACHE_003_ parsing."""
    parser_object = firefox_cache.FirefoxCacheParser()
    storage_writer = self._ParseFile(
        [u'firefox_cache', u'firefox28', u'_CACHE_003_'], parser_object)

    self.assertEqual(len(storage_writer.events), 9)

    event_object = storage_writer.events[7]
    expected_url = (
        u'HTTP:https://ajax.googleapis.com/ajax/libs/jquery/1.8.2/'
        u'jquery.min.js')
    self.assertEqual(event_object.url, expected_url)

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2014-04-21 14:11:07')
    self.assertEqual(event_object.timestamp, expected_timestamp)

    self._VerifyMajorMinor(storage_writer.events)

  @shared_test_lib.skipUnlessHasTestFile([
      u'firefox_cache', u'firefox28', u'E8D65m01'])
  def testParseAlternativeFilename(self):
    """Test Firefox 28 cache 003 file with alternative filename."""
    parser_object = firefox_cache.FirefoxCacheParser()
    storage_writer = self._ParseFile(
        [u'firefox_cache', u'firefox28', u'E8D65m01'], parser_object)

    self.assertEqual(len(storage_writer.events), 9)

  @shared_test_lib.skipUnlessHasTestFile([
      u'firefox_cache', u'firefox3', u'_CACHE_001_'])
  def testParseLegacyCache_001(self):
    """Test Firefox 3 cache file _CACHE_001_ parsing."""
    parser_object = firefox_cache.FirefoxCacheParser()
    storage_writer = self._ParseFile(
        [u'firefox_cache', u'firefox3', u'_CACHE_001_'], parser_object)

    self.assertEqual(len(storage_writer.events), 73)

    event_object = storage_writer.events[0]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2014-05-02 14:15:03')
    self.assertEqual(event_object.timestamp, expected_timestamp)

    expected_msg = (
        u'Fetched 1 time(s) '
        u'[HTTP/1.1 200 OK] GET '
        u'"HTTP:http://start.mozilla.org/en-US/"')
    expected_msg_short = (
        u'[HTTP/1.1 200 OK] GET '
        u'"HTTP:http://start.mozilla.org/en-US/"')

    self._TestGetMessageStrings(
        event_object, expected_msg, expected_msg_short)

  @shared_test_lib.skipUnlessHasTestFile([
      u'firefox_cache', u'firefox3', u'_CACHE_002_'])
  def testParseLegacyCache_002(self):
    """Test Firefox 3 cache file _CACHE_002_ parsing."""
    parser_object = firefox_cache.FirefoxCacheParser()
    storage_writer = self._ParseFile(
        [u'firefox_cache', u'firefox3', u'_CACHE_002_'], parser_object)

    self.assertEqual(len(storage_writer.events), 6)

    event_object = storage_writer.events[2]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2014-05-02 14:25:55')
    self.assertEqual(event_object.timestamp, expected_timestamp)

  @shared_test_lib.skipUnlessHasTestFile([
      u'firefox_cache', u'firefox3', u'_CACHE_003_'])
  def testParseLegacyCache_003(self):
    """Test Firefox 3 cache file _CACHE_003_ parsing."""
    parser_object = firefox_cache.FirefoxCacheParser()
    storage_writer = self._ParseFile(
        [u'firefox_cache', u'firefox3', u'_CACHE_003_'], parser_object)

    self.assertEqual(len(storage_writer.events), 6)

    event_object = storage_writer.events[3]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2014-05-02 14:15:07')
    self.assertEqual(event_object.timestamp, expected_timestamp)


class FirefoxCache2ParserTest(test_lib.ParserTestCase):
  """A unit test for the FirefoxCache2Parser."""

  @shared_test_lib.skipUnlessHasTestFile([
      u'firefox_cache', u'cache2', u'1F4B3A4FC81FB19C530758231FA54313BE8F6FA2'])
  def testParseCache2Entry(self):
    """Test Firefox cache2 file parsing."""
    parser_object = firefox_cache.FirefoxCache2Parser()
    path_segments = [
        u'firefox_cache', u'cache2',
        u'1F4B3A4FC81FB19C530758231FA54313BE8F6FA2']
    storage_writer = self._ParseFile(path_segments, parser_object)

    self.assertEqual(len(storage_writer.events), 3)

    event_object = storage_writer.events[0]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2015-05-02 15:35:31')
    self.assertEqual(event_object.timestamp, expected_timestamp)

    expected_url = (
        u':https://tiles.cdn.mozilla.net/images/'
        u'8acf9436e1b315f5f04b9435a518bcd1aef131f8.5663.png')
    self.assertEqual(event_object.url, expected_url)

    self.assertEqual(event_object.request_method, u'GET')
    self.assertEqual(event_object.response_code, u'HTTP/1.1 200 OK')
    self.assertEqual(event_object.fetch_count, 2)

    event_object = storage_writer.events[1]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2015-05-02 15:35:31')
    self.assertEqual(event_object.timestamp, expected_timestamp)

    event_object = storage_writer.events[2]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2016-05-01 15:35:31')
    self.assertEqual(event_object.timestamp, expected_timestamp)

  @shared_test_lib.skipUnlessHasTestFile([
      u'firefox_cache', u'cache2', u'C966EB70794E44E7E3E8A260106D0C72439AF65B'])
  def testParseInvalidCache2Entry(self):
    """Test file with valid filename and invalid content."""
    parser_object = firefox_cache.FirefoxCache2Parser()
    path_segments = [
        u'firefox_cache', u'cache2',
        u'C966EB70794E44E7E3E8A260106D0C72439AF65B']

    with self.assertRaises(errors.UnableToParseFile):
      self._ParseFile(path_segments, parser_object)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the .customDestinations-ms file parser."""

import unittest

from plaso.formatters import winlnk  # pylint: disable=unused-import
from plaso.lib import eventdata
from plaso.lib import timelib
from plaso.parsers import custom_destinations

from tests import test_lib as shared_test_lib
from tests.parsers import test_lib


class CustomDestinationsParserTest(test_lib.ParserTestCase):
  """Tests for the .customDestinations-ms file parser."""

  @shared_test_lib.skipUnlessHasTestFile([
      u'5afe4de1b92fc382.customDestinations-ms'])
  def testParse(self):
    """Tests the Parse function."""
    parser_object = custom_destinations.CustomDestinationsParser()
    storage_writer = self._ParseFile(
        [u'5afe4de1b92fc382.customDestinations-ms'], parser_object)

    self.assertEqual(len(storage_writer.events), 126)

    # A shortcut event object.
    # The last accessed timestamp.
    event_object = storage_writer.events[121]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2009-07-13 23:55:56.248103')
    self.assertEqual(
        event_object.timestamp_desc, eventdata.EventTimestamp.ACCESS_TIME)
    self.assertEqual(event_object.timestamp, expected_timestamp)

    # The creation timestamp.
    event_object = storage_writer.events[122]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2009-07-13 23:55:56.248103')
    self.assertEqual(
        event_object.timestamp_desc, eventdata.EventTimestamp.CREATION_TIME)
    self.assertEqual(event_object.timestamp, expected_timestamp)

    # The last modification timestamp.
    event_object = storage_writer.events[123]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2009-07-14 01:39:11.388000')
    self.assertEqual(
        event_object.timestamp_desc, eventdata.EventTimestamp.MODIFICATION_TIME)
    self.assertEqual(event_object.timestamp, expected_timestamp)

    expected_msg = (
        u'[@%systemroot%\\system32\\oobefldr.dll,-1262] '
        u'File size: 11776 '
        u'File attribute flags: 0x00000020 '
        u'Drive type: 3 '
        u'Drive serial number: 0x24ba718b '
        u'Local path: C:\\Windows\\System32\\GettingStarted.exe '
        u'cmd arguments: {DE3895CB-077B-4C38-B6E3-F3DE1E0D84FC} '
        u'%systemroot%\\system32\\control.exe /name Microsoft.Display '
        u'env location: %SystemRoot%\\system32\\GettingStarted.exe '
        u'Icon location: %systemroot%\\system32\\display.dll '
        u'Link target: <My Computer> C:\\Windows\\System32\\GettingStarted.exe')

    expected_msg_short = (
        u'[@%systemroot%\\system32\\oobefldr.dll,-1262] '
        u'C:\\Windows\\System32\\GettingStarte...')

    self._TestGetMessageStrings(event_object, expected_msg, expected_msg_short)

    # A shell item event object.
    event_object = storage_writer.events[18]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2010-11-10 07:41:04')
    self.assertEqual(event_object.timestamp, expected_timestamp)

    expected_msg = (
        u'Name: System32 '
        u'Long name: System32 '
        u'NTFS file reference: 2331-1 '
        u'Shell item path: <My Computer> C:\\Windows\\System32 '
        u'Origin: 5afe4de1b92fc382.customDestinations-ms')

    expected_msg_short = (
        u'Name: System32 '
        u'NTFS file reference: 2331-1 '
        u'Origin: 5afe4de1b92fc382.customDes...')

    self._TestGetMessageStrings(event_object, expected_msg, expected_msg_short)

    # A distributed link tracking event object.
    event_object = storage_writer.events[12]

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2010-11-10 19:08:32.656259')
    self.assertEqual(event_object.timestamp, expected_timestamp)

    expected_msg = (
        u'e9215b24-ecfd-11df-a81c-000c29031e1e '
        u'MAC address: 00:0c:29:03:1e:1e '
        u'Origin: 5afe4de1b92fc382.customDestinations-ms')

    expected_msg_short = (
        u'e9215b24-ecfd-11df-a81c-000c29031e1e '
        u'Origin: 5afe4de1b92fc382.customDestinati...')

    self._TestGetMessageStrings(event_object, expected_msg, expected_msg_short)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""This file contains preprocessors for Windows."""

import logging

from plaso.containers import artifacts
from plaso.lib import errors
from plaso.lib import py2to3
from plaso.preprocessors import interface
from plaso.preprocessors import manager
from plaso.winnt import time_zones


class WindowsCodepagePreprocessPlugin(
    interface.WindowsRegistryValuePreprocessPlugin):
  """The Windows codepage preprocess plugin."""

  _REGISTRY_KEY_PATH = (
      u'HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Control\\Nls\\CodePage')
  _REGISTRY_VALUE_NAME = u'ACP'

  def _ParseValueData(self, knowledge_base, value_data):
    """Parses Windows Registry value data for a preprocessing attribute.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      value_data (object): Windows Registry value data.

    Raises:
      errors.PreProcessFail: if the value data is not a string type.
    """
    if not isinstance(value_data, py2to3.UNICODE_TYPE):
      raise errors.PreProcessFail(
          u'Unsupported Registry key: {0:s}, value: {1:s} type: {2:s}.'.format(
              self._REGISTRY_KEY_PATH, self._REGISTRY_VALUE_NAME,
              type(value_data)))

    # Map the Windows code page name to a Python equivalent name.
    codepage = u'cp{0:s}'.format(value_data)

    try:
      knowledge_base.SetCodepage(codepage)
    except ValueError:
      # TODO: add and store preprocessing errors.
      pass


class WindowsHostnamePreprocessPlugin(
    interface.WindowsRegistryValuePreprocessPlugin):
  """The Windows hostname preprocess plugin."""

  _REGISTRY_KEY_PATH = (
      u'HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Control\\ComputerName\\'
      u'ComputerName')
  _REGISTRY_VALUE_NAME = u'ComputerName'

  def _ParseValueData(self, knowledge_base, value_data):
    """Parses Windows Registry value data for a preprocessing attribute.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      value_data (object): Windows Registry value data.

    Raises:
      errors.PreProcessFail: if the value data is not a string type.
    """
    if not isinstance(value_data, py2to3.UNICODE_TYPE):
      raise errors.PreProcessFail(
          u'Unsupported Registry key: {0:s}, value: {1:s} type: {2:s}.'.format(
              self._REGISTRY_KEY_PATH, self._REGISTRY_VALUE_NAME,
              type(value_data)))

    hostname_artifact = artifacts.HostnameArtifact(name=value_data)
    knowledge_base.SetHostname(hostname_artifact)


class WindowsProgramFilesEnvironmentVariable(
    interface.WindowsRegistryEnvironmentVariable):
  """The Windows %ProgramFiles% environment variable preprocess plugin."""

  _NAME = u'programfiles'
  _REGISTRY_KEY_PATH = (
      u'HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows\\CurrentVersion')
  _REGISTRY_VALUE_NAME = u'ProgramFilesDir'


class WindowsProgramFilesX86EnvironmentVariable(
    interface.WindowsRegistryEnvironmentVariable):
  """The Windows %ProgramFilesX86% environment variable preprocess plugin."""

  _NAME = u'programfilesx86'
  _REGISTRY_KEY_PATH = (
      u'HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows\\CurrentVersion')
  _REGISTRY_VALUE_NAME = u'ProgramFilesDir (x86)'


class WindowsSystemProductPlugin(
    interface.WindowsRegistryValuePreprocessPlugin):
  """The Windows system product information preprocess plugin."""

  _REGISTRY_KEY_PATH = (
      u'HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows NT\\CurrentVersion')
  _REGISTRY_VALUE_NAME = u'ProductName'

  def _ParseValueData(self, knowledge_base, value_data):
    """Parses Windows Registry value data for a preprocessing attribute.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      value_data (object): Windows Registry value data.

    Raises:
      errors.PreProcessFail: if the value data is not a string type.
    """
    if not isinstance(value_data, py2to3.UNICODE_TYPE):
      raise errors.PreProcessFail(
          u'Unsupported Registry key: {0:s}, value: {1:s} type: {2:s}.'.format(
              self._REGISTRY_KEY_PATH, self._REGISTRY_VALUE_NAME,
              type(value_data)))

    knowledge_base.SetValue(u'operating_system_product', value_data)


class WindowsSystemRootEnvironmentVariable(
    interface.WindowsPathEnvironmentVariablePlugin):
  """The Windows %SystemRoot% environment variable preprocess plugin."""

  _NAME = u'systemroot'
  _PATH_REGEX = u'/(Windows|WinNT|WINNT35|WTSRV)'


class WindowsSystemVersionPlugin(
    interface.WindowsRegistryValuePreprocessPlugin):
  """The Windows system version information preprocess plugin."""

  _REGISTRY_KEY_PATH = (
      u'HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows NT\\CurrentVersion')
  _REGISTRY_VALUE_NAME = u'CurrentVersion'

  def _ParseValueData(self, knowledge_base, value_data):
    """Parses Windows Registry value data for a preprocessing attribute.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      value_data (object): Windows Registry value data.

    Raises:
      errors.PreProcessFail: if the value data is not a string type.
    """
    if not isinstance(value_data, py2to3.UNICODE_TYPE):
      raise errors.PreProcessFail(
          u'Unsupported Registry key: {0:s}, value: {1:s} type: {2:s}.'.format(
              self._REGISTRY_KEY_PATH, self._REGISTRY_VALUE_NAME,
              type(value_data)))

    knowledge_base.SetValue(u'operating_system_version', value_data)


class WindowsTimeZonePreprocessPlugin(
    interface.WindowsRegistryValuePreprocessPlugin):
  """The Windows time zone preprocess plugin."""

  _REGISTRY_KEY_PATH = (
      u'HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Control\\'
      u'TimeZoneInformation')
  _REGISTRY_VALUE_NAME = u'StandardName'

  def _ParseValueData(self, knowledge_base, value_data):
    """Parses Windows Registry value data for a preprocessing attribute.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      value_data (object): Windows Registry value data.

    Raises:
      errors.PreProcessFail: if the value data is not a string type.
    """
    if not isinstance(value_data, py2to3.UNICODE_TYPE):
      raise errors.PreProcessFail(
          u'Unsupported Registry key: {0:s}, value: {1:s} type: {2:s}.'.format(
              self._REGISTRY_KEY_PATH, self._REGISTRY_VALUE_NAME,
              type(value_data)))

    # Map the Windows time zone name to a Python equivalent name.
    lookup_key = value_data.replace(u' ', u'')

    time_zone = time_zones.TIME_ZONES.get(lookup_key, value_data)
    if not time_zone:
      return

    try:
      # Catch and warn about unsupported mapping.
      knowledge_base.SetTimeZone(time_zone)
    except ValueError:
      # TODO: add and store preprocessing errors.
      time_zone = value_data
      logging.warning(
          u'Unable to map: "{0:s}" to time zone'.format(value_data))


class WindowsUserAccountsPreprocessPlugin(
    interface.WindowsRegistryKeyPreprocessPlugin):
  """The Windows user accounts preprocess plugin."""

  _REGISTRY_KEY_PATH = (
      u'HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows NT\\CurrentVersion\\'
      u'ProfileList')

  def _GetUsernameFromPath(self, path):
    """Retrieves the username from a Windows profile path.

    Trailing path path segment are igored.

    Args:
      path (str): a Windows path with '\\' as path segment separator.

    Returns:
      str: basename which is the last path segment.
    """
    # Strip trailing key separators.
    while path and path[-1] == u'\\':
      path = path[:-1]

    if path:
      _, _, path = path.rpartition(u'\\')
    return path

  def _ParseKey(self, knowledge_base, registry_key):
    """Parses a Windows Registry key for a preprocessing attribute.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      registry_key (WinRegistryKey): Windows Registry key.
    """
    for subkey in registry_key.GetSubkeys():
      if not subkey.name:
        # TODO: add and store preprocessing errors.
        continue

      user_account = artifacts.UserAccountArtifact(
          identifier=subkey.name)

      registry_value = subkey.GetValueByName(u'ProfileImagePath')
      if registry_value:
        profile_path = registry_value.GetDataAsObject()
        username = self._GetUsernameFromPath(profile_path)

        user_account.user_directory = profile_path or None
        user_account.username = username or None

      try:
        knowledge_base.AddUserAccount(user_account)
      except KeyError:
        # TODO: add and store preprocessing errors.
        pass


class WindowsWinDirEnvironmentVariable(
    interface.WindowsPathEnvironmentVariablePlugin):
  """The Windows %WinDir% environment variable preprocess plugin."""

  _NAME = u'windir'
  _PATH_REGEX = u'/(Windows|WinNT|WINNT35|WTSRV)'


manager.PreprocessPluginsManager.RegisterPlugins([
    WindowsCodepagePreprocessPlugin, WindowsHostnamePreprocessPlugin,
    WindowsProgramFilesEnvironmentVariable,
    WindowsProgramFilesX86EnvironmentVariable, WindowsSystemProductPlugin,
    WindowsSystemRootEnvironmentVariable, WindowsSystemVersionPlugin,
    WindowsTimeZonePreprocessPlugin, WindowsUserAccountsPreprocessPlugin,
    WindowsWinDirEnvironmentVariable])



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the image export front-end."""

import os
import unittest

from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.path import factory as path_spec_factory
from dfvfs.resolver import resolver as path_spec_resolver

from plaso.frontend import image_export
from plaso.lib import specification

from tests import test_lib as shared_test_lib
from tests.cli import test_lib as cli_test_lib
from tests.frontend import test_lib


class DateTimeFileEntryFilterTest(shared_test_lib.BaseTestCase):
  """Tests the date time file entry filter."""

  # pylint: disable=protected-access

  def testAddDateTimeRange(self):
    """Tests the AddDateTimeRange function."""
    test_filter = image_export.DateTimeFileEntryFilter()

    test_filter.AddDateTimeRange(
        u'ctime', end_time_string=u'2012-05-25 15:59:25',
        start_time_string=u'2012-05-25 15:59:20')

    with self.assertRaises(ValueError):
      test_filter.AddDateTimeRange(None)

    # Testing adding a badly formatter filter.
    with self.assertRaises(ValueError):
      test_filter.AddDateTimeRange(
          u'foobar', start_time_string=u'2012-02-01 01:01:01')

    with self.assertRaises(ValueError):
      test_filter.AddDateTimeRange(
          u'ctime', end_time_string=u'2012-05-25 15:59:20',
          start_time_string=u'2012-05-25 15:59:25')

    # Testing adding a badly formatter filter, no date set.
    with self.assertRaises(ValueError):
      test_filter.AddDateTimeRange(u'atime')

  @shared_test_lib.skipUnlessHasTestFile([u'ímynd.dd'])
  def testMatches(self):
    """Tests the Matches function."""
    test_path = self._GetTestFilePath([u'ímynd.dd'])
    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_path)
    tsk_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK, inode=16,
        location=u'/a_directory/another_file', parent=os_path_spec)

    file_entry = path_spec_resolver.Resolver.OpenFileEntry(tsk_path_spec)

    # Timestamps of file:
    #   Modified: 2012-05-25T15:59:23+00:00
    #   Accessed: 2012-05-25T15:59:23+00:00
    #    Created: 2012-05-25T15:59:23+00:00

    test_filter = image_export.DateTimeFileEntryFilter()

    # When no date time ranges are specified the filter returns None.
    self.assertIsNone(test_filter.Matches(file_entry))

    # Add a date to the date filter.
    test_filter.AddDateTimeRange(
        u'ctime', start_time_string=u'2012-05-25 15:59:20',
        end_time_string=u'2012-05-25 15:59:25')
    self.assertTrue(test_filter.Matches(file_entry))

    test_filter = image_export.DateTimeFileEntryFilter()
    test_filter.AddDateTimeRange(
        u'ctime', start_time_string=u'2012-05-25 15:59:24',
        end_time_string=u'2012-05-25 15:59:55')
    self.assertFalse(test_filter.Matches(file_entry))

    # Testing a timestamp that does not exist in the stat object.
    test_filter = image_export.DateTimeFileEntryFilter()
    test_filter.AddDateTimeRange(
        u'bkup', start_time_string=u'2012-02-02 12:12:12')
    self.assertTrue(test_filter.Matches(file_entry))

    # Just end date set.
    test_filter = image_export.DateTimeFileEntryFilter()
    test_filter.AddDateTimeRange(
        u'mtime', end_time_string=u'2012-05-25 15:59:55')
    self.assertTrue(test_filter.Matches(file_entry))

    # Just with a start date but within range.
    test_filter = image_export.DateTimeFileEntryFilter()
    test_filter.AddDateTimeRange(
        u'atime', start_time_string=u'2012-03-25 15:59:55')
    self.assertTrue(test_filter.Matches(file_entry))

    # And now with a start date, but out of range.
    test_filter = image_export.DateTimeFileEntryFilter()
    test_filter.AddDateTimeRange(
        u'ctime', start_time_string=u'2012-05-25 15:59:55')
    self.assertFalse(test_filter.Matches(file_entry))

    # Test with more than one date filter.
    test_filter = image_export.DateTimeFileEntryFilter()
    test_filter.AddDateTimeRange(
        u'ctime', start_time_string=u'2012-05-25 15:59:55',
        end_time_string=u'2012-05-25 17:34:12')
    test_filter.AddDateTimeRange(
        u'atime', start_time_string=u'2012-05-25 15:59:20',
        end_time_string=u'2012-05-25 15:59:25')
    test_filter.AddDateTimeRange(
        u'mtime', start_time_string=u'2012-05-25 15:59:24',
        end_time_string=u'2012-05-25 15:59:55')

    self.assertFalse(test_filter.Matches(file_entry))
    self.assertEqual(len(test_filter._date_time_ranges), 3)

  def testPrint(self):
    """Tests the Print function."""
    output_writer = cli_test_lib.TestOutputWriter(encoding=u'utf-8')
    test_filter = image_export.DateTimeFileEntryFilter()

    test_filter.AddDateTimeRange(
        u'ctime', end_time_string=u'2012-05-25 15:59:25',
        start_time_string=u'2012-05-25 15:59:20')

    test_filter.AddDateTimeRange(
        u'atime', end_time_string=u'2012-05-25 15:59:25')

    test_filter.AddDateTimeRange(
        u'mtime', start_time_string=u'2012-05-25 15:59:20')

    test_filter.Print(output_writer)

    expected_output = [
        (b'\tctime between 2012-05-25T15:59:20+00:00 and '
         b'2012-05-25T15:59:25+00:00'),
        b'\tatime after 2012-05-25T15:59:25+00:00',
        b'\tmtime before 2012-05-25T15:59:20+00:00',
        b'']

    output = output_writer.ReadOutput()

    # Compare the output as list of lines which makes it easier to spot
    # differences.
    self.assertEqual(output.split(b'\n'), expected_output)


class ExtensionsFileEntryFilterTest(shared_test_lib.BaseTestCase):
  """Tests the extensions file entry filter."""

  @shared_test_lib.skipUnlessHasTestFile([u'ímynd.dd'])
  def testMatches(self):
    """Tests the Matches function."""
    test_path = self._GetTestFilePath([u'ímynd.dd'])
    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_path)

    test_filter = image_export.ExtensionsFileEntryFilter([u'txt'])

    # Test a filter match.
    tsk_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK, inode=15,
        location=u'/passwords.txt', parent=os_path_spec)

    file_entry = path_spec_resolver.Resolver.OpenFileEntry(tsk_path_spec)
    self.assertTrue(test_filter.Matches(file_entry))

    # Test a filter non-match.
    tsk_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK, inode=16,
        location=u'/a_directory/another_file', parent=os_path_spec)

    file_entry = path_spec_resolver.Resolver.OpenFileEntry(tsk_path_spec)
    self.assertFalse(test_filter.Matches(file_entry))

    # Test that fails because path specification has no location.
    tsk_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK, inode=15, parent=os_path_spec)

    file_entry = path_spec_resolver.Resolver.OpenFileEntry(tsk_path_spec)
    self.assertFalse(test_filter.Matches(file_entry))

  def testPrint(self):
    """Tests the Print function."""
    output_writer = cli_test_lib.TestOutputWriter(encoding=u'utf-8')
    test_filter = image_export.ExtensionsFileEntryFilter([u'exe', u'pdf'])

    test_filter.Print(output_writer)

    expected_output = [
        b'\textensions: exe, pdf',
        b'']

    output = output_writer.ReadOutput()

    # Compare the output as list of lines which makes it easier to spot
    # differences.
    self.assertEqual(output.split(b'\n'), expected_output)


class NamesFileEntryFilterTest(shared_test_lib.BaseTestCase):
  """Tests the names file entry filter."""

  @shared_test_lib.skipUnlessHasTestFile([u'ímynd.dd'])
  def testMatches(self):
    """Tests the Matches function."""
    test_path = self._GetTestFilePath([u'ímynd.dd'])
    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_path)

    test_filter = image_export.NamesFileEntryFilter([u'passwords.txt'])

    # Test a filter non-match.
    tsk_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK, inode=16,
        location=u'/a_directory/another_file', parent=os_path_spec)

    file_entry = path_spec_resolver.Resolver.OpenFileEntry(tsk_path_spec)
    self.assertFalse(test_filter.Matches(file_entry))

    # Test a filter on a directory.
    tsk_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK, inode=12,
        location=u'/a_directory', parent=os_path_spec)

    file_entry = path_spec_resolver.Resolver.OpenFileEntry(tsk_path_spec)
    self.assertFalse(test_filter.Matches(file_entry))

    # Test a filter match.
    tsk_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK, inode=15,
        location=u'/passwords.txt', parent=os_path_spec)

    file_entry = path_spec_resolver.Resolver.OpenFileEntry(tsk_path_spec)
    self.assertTrue(test_filter.Matches(file_entry))

    # Test a filter without names.
    test_filter = image_export.NamesFileEntryFilter([])
    self.assertFalse(test_filter.Matches(file_entry))

  def testPrint(self):
    """Tests the Print function."""
    output_writer = cli_test_lib.TestOutputWriter(encoding=u'utf-8')
    test_filter = image_export.NamesFileEntryFilter([u'myfile'])

    test_filter.Print(output_writer)

    expected_output = [
        b'\tnames: myfile',
        b'']

    output = output_writer.ReadOutput()

    # Compare the output as list of lines which makes it easier to spot
    # differences.
    self.assertEqual(output.split(b'\n'), expected_output)


class SignaturesFileEntryFilterTest(shared_test_lib.BaseTestCase):
  """Tests the signatures file entry filter."""

  # pylint: disable=protected-access

  def testGetScanner(self):
    """Tests the _GetScanner function."""
    test_filter = image_export.SignaturesFileEntryFilter(None, [])

    test_filter._GetScanner(None, [])
    self.assertIsNone(test_filter._file_scanner)

    specification_store = specification.FormatSpecificationStore()
    format_specification = specification.FormatSpecification(u'no_offset')
    format_specification.AddNewSignature(b'test1')
    specification_store.AddSpecification(format_specification)

    format_specification = specification.FormatSpecification(u'negative_offset')
    format_specification.AddNewSignature(b'test2', offset=-4)
    specification_store.AddSpecification(format_specification)

    format_specification = specification.FormatSpecification(u'positive_offset')
    format_specification.AddNewSignature(b'test3', offset=4)
    specification_store.AddSpecification(format_specification)

    with self.assertRaises(TypeError):
      # Currently pysigscan does not support patterns without an offset.
      test_filter._GetScanner(specification_store, [u'no_offset'])

    file_scanner = test_filter._GetScanner(
        specification_store, [u'negative_offset'])
    self.assertIsNotNone(file_scanner)

    file_scanner = test_filter._GetScanner(
        specification_store, [u'positive_offset'])
    self.assertIsNotNone(file_scanner)

  @shared_test_lib.skipUnlessHasTestFile([u'NTUSER.DAT'])
  @shared_test_lib.skipUnlessHasTestFile([u'test_pe.exe'])
  def testMatches(self):
    """Tests the Matches function."""
    specification_store = specification.FormatSpecificationStore()
    format_specification = specification.FormatSpecification(u'regf')
    format_specification.AddNewSignature(b'regf', offset=0)
    specification_store.AddSpecification(format_specification)

    test_filter = image_export.SignaturesFileEntryFilter(
        specification_store, [u'regf'])

    # Test a filter match.
    test_path = self._GetTestFilePath([u'NTUSER.DAT'])
    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_path)

    file_entry = path_spec_resolver.Resolver.OpenFileEntry(os_path_spec)
    self.assertTrue(test_filter.Matches(file_entry))

    # Test a filter non-match.
    test_path = self._GetTestFilePath([u'test_pe.exe'])
    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_path)

    file_entry = path_spec_resolver.Resolver.OpenFileEntry(os_path_spec)
    self.assertFalse(test_filter.Matches(file_entry))

  def testPrint(self):
    """Tests the Print function."""
    output_writer = cli_test_lib.TestOutputWriter(encoding=u'utf-8')

    specification_store = specification.FormatSpecificationStore()
    specification_store.AddNewSpecification(u'7z')

    test_filter = image_export.SignaturesFileEntryFilter(
        specification_store, [u'7z', u'bzip2'])

    test_filter.Print(output_writer)

    expected_output = [
        b'\tsignature identifiers: 7z',
        b'']

    output = output_writer.ReadOutput()

    # Compare the output as list of lines which makes it easier to spot
    # differences.
    self.assertEqual(output.split(b'\n'), expected_output)


class FileEntryFilterCollectionTest(shared_test_lib.BaseTestCase):
  """Tests the file entry filter collection."""

  # pylint: disable=protected-access

  def testAddFilter(self):
    """Tests the AddFilter function."""
    test_filter_collection = image_export.FileEntryFilterCollection()

    self.assertEqual(len(test_filter_collection._filters), 0)

    file_entry_filter = image_export.NamesFileEntryFilter([u'name'])
    test_filter_collection.AddFilter(file_entry_filter)
    self.assertEqual(len(test_filter_collection._filters), 1)

  def testHasFilters(self):
    """Tests the HasFilters function."""
    test_filter_collection = image_export.FileEntryFilterCollection()
    self.assertFalse(test_filter_collection.HasFilters())

    test_filter_collection = image_export.FileEntryFilterCollection()
    file_entry_filter = image_export.NamesFileEntryFilter([u'name'])
    test_filter_collection.AddFilter(file_entry_filter)
    self.assertTrue(test_filter_collection.HasFilters())

  # TODO: add test for Matches.
  # TODO: add test for Print.


class ImageExportFrontendTest(shared_test_lib.BaseTestCase):
  """Tests the image export front-end."""

  # pylint: disable=protected-access

  def _GetTestScanNode(self, scan_context):
    """Retrieves the scan node for testing.

    Retrieves the first scan node, from the root upwards, with more or less
    than 1 sub node.

    Args:
      scan_context (dfvfs.SourceScanContext): scan context.

    Returns:
      dfvfs.SourceScanNode: scan node.
    """
    scan_node = scan_context.GetRootScanNode()
    while len(scan_node.sub_nodes) == 1:
      scan_node = scan_node.sub_nodes[0]

    return scan_node

  def _RecursiveList(self, path):
    """Recursively lists a file or directory.

    Args:
      path (str): path of the file or directory to list.

    Returns:
      list[str]: names of files and sub directories within the path.
    """
    results = []
    for sub_path, _, files in os.walk(path):
      if sub_path != path:
        results.append(sub_path)

      for file_entry in files:
        results.append(os.path.join(sub_path, file_entry))

    return results

  @shared_test_lib.skipUnlessHasTestFile([u'ímynd.dd'])
  def testCalculateDigestHash(self):
    """Tests the _CalculateDigestHash function."""
    test_front_end = image_export.ImageExportFrontend()

    test_path = self._GetTestFilePath([u'ímynd.dd'])
    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_path)
    tsk_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK, inode=16,
        location=u'/a_directory/another_file', parent=os_path_spec)

    file_entry = path_spec_resolver.Resolver.OpenFileEntry(tsk_path_spec)
    digest_hash = test_front_end._CalculateDigestHash(file_entry, u'')
    expected_digest_hash = (
        u'c7fbc0e821c0871805a99584c6a384533909f68a6bbe9a2a687d28d9f3b10c16')
    self.assertEqual(digest_hash, expected_digest_hash)

    test_path = self._GetTestFilePath([u'ímynd.dd'])
    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_path)
    tsk_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK, inode=12,
        location=u'/a_directory', parent=os_path_spec)

    file_entry = path_spec_resolver.Resolver.OpenFileEntry(tsk_path_spec)
    with self.assertRaises(IOError):
      test_front_end._CalculateDigestHash(file_entry, u'')

  # TODO: add test for _CreateSanitizedDestinationDirectory.
  # TODO: add test for _Extract.

  @shared_test_lib.skipUnlessHasTestFile([u'ímynd.dd'])
  def testExtractDataStream(self):
    """Tests the _ExtractDataStream function."""
    output_writer = cli_test_lib.TestOutputWriter(encoding=u'utf-8')
    test_front_end = image_export.ImageExportFrontend()

    test_path = self._GetTestFilePath([u'ímynd.dd'])
    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_path)
    tsk_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK, inode=16,
        location=u'/a_directory/another_file', parent=os_path_spec)

    file_entry = path_spec_resolver.Resolver.OpenFileEntry(tsk_path_spec)
    with shared_test_lib.TempDirectory() as temp_directory:
      test_front_end._ExtractDataStream(
          file_entry, u'', temp_directory, output_writer)

  @shared_test_lib.skipUnlessHasTestFile([u'ímynd.dd'])
  def testExtractFileEntry(self):
    """Tests the _ExtractFileEntry function."""
    output_writer = cli_test_lib.TestOutputWriter(encoding=u'utf-8')
    test_front_end = image_export.ImageExportFrontend()

    test_path = self._GetTestFilePath([u'ímynd.dd'])
    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_path)
    tsk_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK, inode=16,
        location=u'/a_directory/another_file', parent=os_path_spec)

    with shared_test_lib.TempDirectory() as temp_directory:
      test_front_end._ExtractFileEntry(
          tsk_path_spec, temp_directory, output_writer)

  # TODO: add test for _ExtractWithFilter.
  # TODO: add test for _GetSourceFileSystem.
  # TODO: add test for _Preprocess.

  @shared_test_lib.skipUnlessHasTestFile([u'ímynd.dd'])
  def testWriteFileEntry(self):
    """Tests the _WriteFileEntry function."""
    test_front_end = image_export.ImageExportFrontend()

    test_path = self._GetTestFilePath([u'ímynd.dd'])
    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_path)
    tsk_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK, inode=16,
        location=u'/a_directory/another_file', parent=os_path_spec)

    file_entry = path_spec_resolver.Resolver.OpenFileEntry(tsk_path_spec)
    with shared_test_lib.TempDirectory() as temp_directory:
      destination_path = os.path.join(temp_directory, u'another_file')
      test_front_end._WriteFileEntry(file_entry, u'', destination_path)

  def testHasFilters(self):
    """Tests the HasFilters function."""
    test_front_end = image_export.ImageExportFrontend()
    self.assertFalse(test_front_end.HasFilters())

    test_front_end = image_export.ImageExportFrontend()
    test_front_end.ParseDateFilters([
        u'ctime, 2012-05-25 15:59:00, 2012-05-25 15:59:20'])
    self.assertTrue(test_front_end.HasFilters())

    test_front_end = image_export.ImageExportFrontend()
    test_front_end.ParseExtensionsString(u'txt')
    self.assertTrue(test_front_end.HasFilters())

    test_front_end = image_export.ImageExportFrontend()
    test_front_end.ParseNamesString(u'another_file')
    self.assertTrue(test_front_end.HasFilters())

    test_front_end = image_export.ImageExportFrontend()
    test_front_end.ParseSignatureIdentifiers(self._DATA_PATH, u'gzip')
    self.assertTrue(test_front_end.HasFilters())

  def testParseDateFilters(self):
    """Tests the ParseDateFilters function."""
    test_front_end = image_export.ImageExportFrontend()

    test_front_end.ParseDateFilters([
        u'ctime, 2012-05-25 15:59:00, 2012-05-25 15:59:20'])

    with self.assertRaises(ValueError):
      test_front_end.ParseDateFilters([u'ctime, 2012-05-25 15:59:00'])

    with self.assertRaises(ValueError):
      test_front_end.ParseDateFilters([
          u'ctime, 2012-05-25 15:59:00, 2012-05-A5 15:59:20'])

  def testParseExtensionsString(self):
    """Tests the ParseExtensionsString function."""
    test_front_end = image_export.ImageExportFrontend()

    test_front_end.ParseExtensionsString(u'txt')

  def testParseNamesString(self):
    """Tests the ParseNamesString function."""
    test_front_end = image_export.ImageExportFrontend()

    test_front_end.ParseNamesString(u'another_file')

  def testParseSignatureIdentifiers(self):
    """Tests the ParseSignatureIdentifiers function."""
    test_front_end = image_export.ImageExportFrontend()

    test_front_end.ParseSignatureIdentifiers(self._DATA_PATH, u'gzip')

    with self.assertRaises(ValueError):
      test_front_end.ParseSignatureIdentifiers(None, u'gzip')

    with self.assertRaises(IOError):
      test_path = os.path.join(os.sep, u'bogus')
      test_front_end.ParseSignatureIdentifiers(test_path, u'gzip')

  # TODO: add test for PrintFilterCollection.

  @shared_test_lib.skipUnlessHasTestFile([u'image.qcow2'])
  def testProcessSourcesExtractWithDateTimeFilter(self):
    """Tests the ProcessSources function with a date time filter."""
    output_writer = cli_test_lib.TestOutputWriter(encoding=u'utf-8')
    test_front_end = image_export.ImageExportFrontend()
    test_front_end.ParseDateFilters([
        u'ctime, 2012-05-25 15:59:00, 2012-05-25 15:59:20'])

    # TODO: move to corresponding CLI test.
    output_writer = test_lib.StringIOOutputWriter()
    test_front_end.PrintFilterCollection(output_writer)

    expected_value = (
        u'Filters:\n'
        u'\tctime between 2012-05-25T15:59:00+00:00 and '
        u'2012-05-25T15:59:20+00:00\n')
    value = output_writer.GetValue()
    self.assertEqual(value, expected_value)

    test_path = self._GetTestFilePath([u'image.qcow2'])
    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_path)
    qcow_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_QCOW, parent=os_path_spec)
    path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK, location=u'/',
        parent=qcow_path_spec)

    with shared_test_lib.TempDirectory() as temp_directory:
      test_front_end.ProcessSources([path_spec], temp_directory, output_writer)

      expected_extracted_files = sorted([
          os.path.join(temp_directory, u'a_directory'),
          os.path.join(temp_directory, u'a_directory', u'a_file')])

      extracted_files = self._RecursiveList(temp_directory)

    self.assertEqual(sorted(extracted_files), expected_extracted_files)

  @shared_test_lib.skipUnlessHasTestFile([u'image.qcow2'])
  def testProcessSourcesExtractWithExtensionsFilter(self):
    """Tests the ProcessSources function with an extensions filter."""
    output_writer = cli_test_lib.TestOutputWriter(encoding=u'utf-8')
    test_front_end = image_export.ImageExportFrontend()
    test_front_end.ParseExtensionsString(u'txt')

    test_path = self._GetTestFilePath([u'image.qcow2'])
    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_path)
    qcow_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_QCOW, parent=os_path_spec)
    path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK, location=u'/',
        parent=qcow_path_spec)

    with shared_test_lib.TempDirectory() as temp_directory:
      test_front_end.ProcessSources([path_spec], temp_directory, output_writer)

      expected_extracted_files = sorted([
          os.path.join(temp_directory, u'passwords.txt')])

      extracted_files = self._RecursiveList(temp_directory)

    self.assertEqual(sorted(extracted_files), expected_extracted_files)

  @shared_test_lib.skipUnlessHasTestFile([u'image.qcow2'])
  def testProcessSourcesExtractWithNamesFilter(self):
    """Tests the ProcessSources function with a names filter."""
    output_writer = cli_test_lib.TestOutputWriter(encoding=u'utf-8')
    test_front_end = image_export.ImageExportFrontend()
    test_front_end.ParseNamesString(u'another_file')

    test_path = self._GetTestFilePath([u'image.qcow2'])
    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_path)
    qcow_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_QCOW, parent=os_path_spec)
    path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK, location=u'/',
        parent=qcow_path_spec)

    with shared_test_lib.TempDirectory() as temp_directory:
      test_front_end.ProcessSources([path_spec], temp_directory, output_writer)

      expected_extracted_files = sorted([
          os.path.join(temp_directory, u'a_directory'),
          os.path.join(temp_directory, u'a_directory', u'another_file')])

      extracted_files = self._RecursiveList(temp_directory)

    self.assertEqual(sorted(extracted_files), expected_extracted_files)

  @shared_test_lib.skipUnlessHasTestFile([u'image.qcow2'])
  def testProcessSourcesExtractWithFilter(self):
    """Tests the ProcessSources function with a filter file."""
    output_writer = cli_test_lib.TestOutputWriter(encoding=u'utf-8')
    test_front_end = image_export.ImageExportFrontend()

    test_path = self._GetTestFilePath([u'image.qcow2'])
    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_path)
    qcow_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_QCOW, parent=os_path_spec)
    path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK, location=u'/',
        parent=qcow_path_spec)

    with shared_test_lib.TempDirectory() as temp_directory:
      filter_file = os.path.join(temp_directory, u'filter.txt')
      with open(filter_file, 'wb') as file_object:
        file_object.write(b'/a_directory/.+_file\n')

      test_front_end.ProcessSources(
          [path_spec], temp_directory, output_writer, filter_file=filter_file)

      expected_extracted_files = sorted([
          os.path.join(temp_directory, u'filter.txt'),
          os.path.join(temp_directory, u'a_directory'),
          os.path.join(temp_directory, u'a_directory', u'another_file'),
          os.path.join(temp_directory, u'a_directory', u'a_file')])

      extracted_files = self._RecursiveList(temp_directory)

    self.assertEqual(sorted(extracted_files), expected_extracted_files)

  @shared_test_lib.skipUnlessHasTestFile([u'syslog_image.dd'])
  def testProcessSourcesExtractWithSignaturesFilter(self):
    """Tests the ProcessSources function with a signatures filter."""
    output_writer = cli_test_lib.TestOutputWriter(encoding=u'utf-8')
    test_front_end = image_export.ImageExportFrontend()
    test_front_end.ParseSignatureIdentifiers(self._DATA_PATH, u'gzip')

    test_path = self._GetTestFilePath([u'syslog_image.dd'])
    os_path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=test_path)
    path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_TSK, location=u'/',
        parent=os_path_spec)

    with shared_test_lib.TempDirectory() as temp_directory:
      test_front_end.ProcessSources([path_spec], temp_directory, output_writer)

      expected_extracted_files = sorted([
          os.path.join(temp_directory, u'logs'),
          os.path.join(temp_directory, u'logs', u'sys.tgz')])

      extracted_files = self._RecursiveList(temp_directory)

    self.assertEqual(sorted(extracted_files), expected_extracted_files)

  # TODO: add test with remove duplicates disabled.
  # TODO: add test for ReadSpecificationFile.


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Windows Prefetch event formatter."""

import unittest

from plaso.formatters import winprefetch

from tests.formatters import test_lib


class WinPrefetchExecutionFormatterTest(test_lib.EventFormatterTestCase):
  """Tests for the Windows Prefetch execution event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = winprefetch.WinPrefetchExecutionFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = winprefetch.WinPrefetchExecutionFormatter()

    expected_attribute_names = [
        u'executable',
        u'run_count',
        u'path',
        u'prefetch_hash',
        u'volumes_string']

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)

  # TODO: add test for GetMessages.


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the bencode parser plugin for uTorrent files."""

import unittest

from plaso.formatters import bencode_parser  # pylint: disable=unused-import
from plaso.lib import eventdata
from plaso.lib import timelib
from plaso.parsers import bencode_parser

from tests import test_lib as shared_test_lib
from tests.parsers.bencode_plugins import test_lib


class UTorrentPluginTest(test_lib.BencodePluginTestCase):
  """Tests for bencode parser plugin for uTorrent files."""

  @shared_test_lib.skipUnlessHasTestFile([u'bencode_utorrent'])
  def testProcess(self):
    """Tests the Process function."""
    parser_object = bencode_parser.BencodeParser()
    storage_writer = self._ParseFile([u'bencode_utorrent'], parser_object)

    self.assertEqual(len(storage_writer.events), 4)

    # The order in which BencodeParser generates events is nondeterministic
    # hence we sort the events.
    events = self._GetSortedEvents(storage_writer.events)

    expected_caption = u'plaso test'
    expected_path = u'e:\\torrent\\files\\plaso test'

    # First test on when the torrent was added to the client.
    event_object = events[0]

    self.assertEqual(event_object.caption, expected_caption)
    self.assertEqual(event_object.path, expected_path)
    self.assertEqual(event_object.seedtime, 511)

    expected_description = eventdata.EventTimestamp.ADDED_TIME
    self.assertEqual(event_object.timestamp_desc, expected_description)

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-08-03 14:52:12')
    self.assertEqual(event_object.timestamp, expected_timestamp)

    # Second test on when the torrent file was completely downloaded.
    event_object = events[3]

    self.assertEqual(event_object.caption, expected_caption)
    self.assertEqual(event_object.path, expected_path)
    self.assertEqual(event_object.seedtime, 511)

    expected_description = eventdata.EventTimestamp.FILE_DOWNLOADED
    self.assertEqual(event_object.timestamp_desc, expected_description)

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-08-03 18:11:35')
    self.assertEqual(event_object.timestamp, expected_timestamp)

    # Third test on when the torrent was first modified.
    event_object = events[2]

    self.assertEqual(event_object.caption, expected_caption)
    self.assertEqual(event_object.path, expected_path)
    self.assertEqual(event_object.seedtime, 511)

    expected_description = eventdata.EventTimestamp.MODIFICATION_TIME
    self.assertEqual(event_object.timestamp_desc, expected_description)

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-08-03 18:11:34')
    self.assertEqual(event_object.timestamp, expected_timestamp)

    # Fourth test on when the torrent was again modified.
    event_object = events[1]

    self.assertEqual(event_object.caption, expected_caption)
    self.assertEqual(event_object.path, expected_path)
    self.assertEqual(event_object.seedtime, 511)

    expected_description = eventdata.EventTimestamp.MODIFICATION_TIME
    self.assertEqual(event_object.timestamp_desc, expected_description)

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2013-08-03 16:27:59')
    self.assertEqual(event_object.timestamp, expected_timestamp)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Linux preprocess plug-ins."""

import unittest

from dfvfs.helpers import fake_file_system_builder
from dfvfs.path import fake_path_spec

from plaso.preprocessors import linux

from tests.preprocessors import test_lib


class LinuxHostnamePreprocessPluginTest(test_lib.PreprocessPluginTestCase):
  """Tests for the Linux hostname preprocess plug-in object."""

  _FILE_DATA = 'plaso.kiddaland.net\n'

  def testRun(self):
    """Tests the Run function."""
    file_system_builder = fake_file_system_builder.FakeFileSystemBuilder()
    file_system_builder.AddFile(u'/etc/hostname', self._FILE_DATA)

    mount_point = fake_path_spec.FakePathSpec(location=u'/')

    plugin = linux.LinuxHostnamePreprocessPlugin()
    knowledge_base = self._RunFileSystemPlugin(
        file_system_builder.file_system, mount_point, plugin)

    self.assertEqual(knowledge_base.hostname, u'plaso.kiddaland.net')


class LinuxTimeZonePreprocessPluginTest(test_lib.PreprocessPluginTestCase):
  """Test for the Linux timezone preprocess plug-in object."""

  _FILE_DATA = 'Europe/Zurich\n'

  def testRun(self):
    """Test the Run function."""
    file_system_builder = fake_file_system_builder.FakeFileSystemBuilder()
    file_system_builder.AddFile(u'/etc/timezone', self._FILE_DATA)

    mount_point = fake_path_spec.FakePathSpec(location=u'/')

    plugin = linux.LinuxTimeZonePreprocessPlugin()
    knowledge_base = self._RunFileSystemPlugin(
        file_system_builder.file_system, mount_point, plugin)

    self.assertEqual(knowledge_base.timezone.zone, u'Europe/Zurich')


class LinuxUserAccountsPreprocessPluginTest(test_lib.PreprocessPluginTestCase):
  """Tests for the Linux usernames preprocess plug-in object."""

  _FILE_DATA = (
      'root:x:0:0:root:/root:/bin/bash\n'
      'bin:x:1:1:bin:/bin:/sbin/nologin\n'
      'daemon:x:2:2:daemon:/sbin:/sbin/nologin\n'
      'adm:x:3:4:adm:/var/adm:/sbin/nologin\n'
      'lp:x:4:7:lp:/var/spool/lpd:/sbin/nologin\n'
      'sync:x:5:0:sync:/sbin:/bin/sync\n'
      'shutdown:x:6:0:shutdown:/sbin:/sbin/shutdown\n'
      'halt:x:7:0:halt:/sbin:/sbin/halt\n'
      'mail:x:8:12:mail:/var/spool/mail:/sbin/nologin\n'
      'operator:x:11:0:operator:/root:/sbin/nologin\n'
      'games:x:12:100:games:/usr/games:/sbin/nologin\n'
      'ftp:x:14:50:FTP User:/var/ftp:/sbin/nologin\n'
      'nobody:x:99:99:Nobody:/:/sbin/nologin\n')

  def testRun(self):
    """Tests the Run function."""
    file_system_builder = fake_file_system_builder.FakeFileSystemBuilder()
    file_system_builder.AddFile(u'/etc/passwd', self._FILE_DATA)

    mount_point = fake_path_spec.FakePathSpec(location=u'/')

    plugin = linux.LinuxUserAccountsPreprocessPlugin()
    knowledge_base = self._RunFileSystemPlugin(
        file_system_builder.file_system, mount_point, plugin)

    users = sorted(
        knowledge_base.user_accounts,
        key=lambda user_account: user_account.identifier)
    self.assertEqual(len(users), 13)

    user_account = users[4]

    self.assertEqual(user_account.identifier, u'14')
    self.assertEqual(user_account.group_identifier, u'50')
    self.assertEqual(user_account.user_directory, u'/var/ftp')
    self.assertEqual(user_account.username, u'ftp')
    self.assertEqual(user_account.shell, u'/sbin/nologin')


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""This file contains the Spotlight searched terms plugin in Plaso."""

from plaso.containers import plist_event
from plaso.parsers import plist
from plaso.parsers.plist_plugins import interface


__author__ = 'Joaquin Moreno Garijo (Joaquin.MorenoGarijo.2013@live.rhul.ac.uk)'


class SpotlightPlugin(interface.PlistPlugin):
  """Basic plugin to extract information from Spotlight plist file.

  Further information about extracted fields:
    name of the item:
      search term.

    PATH:
      path of the program associated to the term.

    LAST_USED:
      last time when it was executed.

    DISPLAY_NAME:
      the display name of the program associated.
  """

  NAME = u'spotlight'
  DESCRIPTION = u'Parser for Spotlight plist files.'

  PLIST_PATH = u'com.apple.spotlight.plist'
  PLIST_KEYS = frozenset([u'UserShortcuts'])

  def GetEntries(self, parser_mediator, match=None, **unused_kwargs):
    """Extracts relevant Spotlight entries.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      match (Optional[dict[str: object]]): keys extracted from PLIST_KEYS.
    """
    shortcuts = match.get(u'UserShortcuts', {})
    for search_text, data in iter(shortcuts.items()):
      if not u'LAST_USED' in data:
        continue
      description = (
          u'Spotlight term searched "{0:s}" associate to {1:s} '
          u'({2:s})').format(
              search_text, data.get(u'DISPLAY_NAME', u'<DISPLAY_NAME>'),
              data.get(u'PATH', u'<PATH>'))
      event_object = plist_event.PlistEvent(
          u'/UserShortcuts', search_text, data[u'LAST_USED'], description)
      parser_mediator.ProduceEvent(event_object)


plist.PlistParser.RegisterPlugin(SpotlightPlugin)



# -*- coding: utf-8 -*-
"""Shared functions and classes for testing."""

import os
import shutil
import sys
import tempfile
import unittest

from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.path import factory as path_spec_factory
from dfvfs.resolver import resolver as path_spec_resolver


def skipUnlessHasTestFile(path_segments):
  """Decorator to skip a test if the test file does not exist.

  Args:
    path_segments (list[str]): path segments inside the test data directory.

  Returns:
    function: to invoke.
  """
  fail_unless_has_test_file = getattr(
      unittest, u'fail_unless_has_test_file', False)

  path = os.path.join(u'test_data', *path_segments)
  if fail_unless_has_test_file or os.path.exists(path):
    return lambda function: function

  if sys.version_info[0] < 3:
    path = path.encode(u'utf-8')

  # Note that the message should be of type str which is different for
  # different versions of Python.
  return unittest.skip('missing test file: {0:s}'.format(path))


def GetTestFilePath(path_segments):
  """Retrieves the path of a test file in the test data directory.

  Args:
    path_segments (list[str]): path segments inside the test data directory.

  Returns:
    str: path of the test file.
  """
  # Note that we need to pass the individual path segments to os.path.join
  # and not a list.
  return os.path.join(os.getcwd(), u'test_data', *path_segments)


class BaseTestCase(unittest.TestCase):
  """The base test case."""

  _DATA_PATH = os.path.join(os.getcwd(), u'data')
  _TEST_DATA_PATH = os.path.join(os.getcwd(), u'test_data')

  # Show full diff results, part of TestCase so does not follow our naming
  # conventions.
  maxDiff = None

  def _GetTestFileEntry(self, path_segments):
    """Creates a file entry that references a file in the test data directory.

    Args:
      path_segments (list[str]): path segments inside the test data directory.

    Returns:
      dfvfs.FileEntry: file entry.
    """
    path = self._GetTestFilePath(path_segments)
    path_spec = path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=path)
    return path_spec_resolver.Resolver.OpenFileEntry(path_spec)

  def _GetTestFilePath(self, path_segments):
    """Retrieves the path of a test file in the test data directory.

    Args:
      path_segments (list[str]): path segments inside the test data directory.

    Returns:
      str: path of the test file.
    """
    # Note that we need to pass the individual path segments to os.path.join
    # and not a list.
    return os.path.join(self._TEST_DATA_PATH, *path_segments)

  def _GetTestFilePathSpec(self, path_segments):
    """Retrieves a path specification of a test file in the test data directory.

    Args:
      path_segments (list[str]): components of a path to a test file, relative
          to the test_data directory.

    Returns:
      dfvfs.PathSpec: path specification.
    """
    path = self._GetTestFilePath(path_segments)
    return path_spec_factory.Factory.NewPathSpec(
        dfvfs_definitions.TYPE_INDICATOR_OS, location=path)


class TempDirectory(object):
  """Class that implements a temporary directory."""

  def __init__(self):
    """Initializes a temporary directory."""
    super(TempDirectory, self).__init__()
    self.name = u''

  def __enter__(self):
    """Make this work with the 'with' statement."""
    self.name = tempfile.mkdtemp()
    return self.name

  def __exit__(self, unused_type, unused_value, unused_traceback):
    """Make this work with the 'with' statement."""
    shutil.rmtree(self.name, True)



# -*- coding: utf-8 -*-
"""This file contains the event buffer class."""

import heapq
import logging

from plaso.lib import errors
from plaso.lib import py2to3


class _EventsHeap(object):
  """Class that defines the events heap."""

  def __init__(self):
    """Initializes an events heap."""
    super(_EventsHeap, self).__init__()
    self._heap = []

  @property
  def number_of_events(self):
    """int: number of serialized events on the heap."""
    return len(self._heap)

  def PopEvent(self):
    """Pops an event from the heap.

    Returns:
      EventObject: event.
    """
    try:
      _, _, _, event = heapq.heappop(self._heap)
      return event

    except IndexError:
      return None

  def PushEvent(self, event):
    """Pushes an event onto the heap.

    Args:
      event (EventObject): event.
    """
    event_identifier = event.GetIdentifier()
    event_identifier_string = event_identifier.CopyToString()
    heap_values = (
        event.timestamp, event.timestamp_desc, event_identifier_string, event)
    heapq.heappush(self._heap, heap_values)

  def PushEvents(self, events):
    """Pushes events onto the heap.

    Args:
      events list[EventObject]: events.
    """
    for event in events:
      self.PushEvent(event)


# TODO: rename class and fix docstrings.
class EventBuffer(object):
  """Buffer class for event output processing.

  The event buffer is used to deduplicate events and make sure they are sorted
  before output.

  Attributes:
    check_dedups (bool): True if the event buffer should check and merge
        duplicate events.
    duplicate_counter (int): number of duplicate events.
  """

  _JOIN_ATTRIBUTES = frozenset([u'display_name', u'filename', u'inode'])

  # Attributes that should not be used in calculating the event key.
  # TODO: remove uuid when test files have been updated.
  _EXCLUDED_ATTRIBUTES = frozenset([
      u'data_type',
      u'display_name',
      u'filename',
      u'inode',
      u'parser',
      u'pathspec',
      u'tag',
      u'timestamp'])

  def __init__(self, output_module, check_dedups=True):
    """Initializes an event buffer object.

    This class is used for buffering up events for duplicate removals
    and for other post-processing/analysis of events before being presented
    by the appropriate output module.

    Args:
      output_module (OutputModule): output module.
      check_dedups (Optional[bool]): True if the event buffer should check and
          merge duplicate events.
    """
    self._current_timestamp = 0
    self._events_per_key = {}
    self._output_module = output_module
    self._output_module.Open()
    self._output_module.WriteHeader()

    self.check_dedups = check_dedups
    self.duplicate_counter = 0

  def __enter__(self):
    """Make usable with "with" statement."""
    return self

  def __exit__(self, unused_type, unused_value, unused_traceback):
    """Make usable with "with" statement."""
    self.End()

  def _GetEventIdentifier(self, event):
    """Determines an unique identifier of an event from its attributes.

    Args:
      event (EventObject): event.

    Returns:
      str: unique identifier representation of the event that can be used for
          equality comparison.
    """
    attributes = {}
    for attribute_name, attribute_value in event.GetAttributes():
      if attribute_name in self._EXCLUDED_ATTRIBUTES:
        continue

      if isinstance(attribute_value, dict):
        attribute_value = sorted(attribute_value.items())

      elif isinstance(attribute_value, set):
        attribute_value = sorted(list(attribute_value))

      attributes[attribute_name] = attribute_value

    if event.pathspec:
      attributes[u'pathspec'] = event.pathspec.comparable

    try:
      event_identifier_string = u'|'.join([
          u'{0:s}={1!s}'.format(attribute_name, attribute_value)
          for attribute_name, attribute_value in sorted(attributes.items())])

    except UnicodeDecodeError:
      event_identifier = event.GetIdentifier()
      event_identifier_string = u'identifier={0:s}'.format(
          event_identifier.CopyToString())

    event_identifier_string = u'{0:d}|{1:s}|{2:s}'.format(
        event.timestamp, event.data_type, event_identifier_string)
    return event_identifier_string

  def Append(self, event):
    """Appends an event.

    Args:
      event (EventObject): event.
    """
    if not self.check_dedups:
      self._output_module.WriteEvent(event)
      return

    if event.timestamp != self._current_timestamp:
      self._current_timestamp = event.timestamp
      self.Flush()

    lookup_key = self._GetEventIdentifier(event)
    if lookup_key in self._events_per_key:
      duplicate_event = self._events_per_key.pop(lookup_key)
      self.JoinEvents(event, duplicate_event)

    self._events_per_key[lookup_key] = event

  def End(self):
    """Closes the buffer.

    Buffered events are written using the output module, an optional footer is
    written and the output is closed.
    """
    self.Flush()

    if self._output_module:
      self._output_module.WriteFooter()
      self._output_module.Close()

  def Flush(self):
    """Flushes the buffer.

    Buffered events are written using the output module.
    """
    if not self._events_per_key:
      return

    # The heap is used to make sure the events are sorted in
    # a deterministic way.
    events_heap = _EventsHeap()
    events_heap.PushEvents(self._events_per_key.values())
    self._events_per_key = {}

    event = events_heap.PopEvent()
    while event:
      try:
        self._output_module.WriteEvent(event)
      except errors.WrongFormatter as exception:
        # TODO: store errors and report them at the end of psort.
        logging.error(
            u'Unable to write event with error: {0:s}'.format(exception))

      event = events_heap.PopEvent()

  def JoinEvents(self, first_event, second_event):
    """Joins the attributes of two events.

    Args:
      first_event (EventObject): first event.
      second_event (EventObject): second event.
    """
    self.duplicate_counter += 1

    # TODO: Currently we are using the first event pathspec, perhaps that
    # is not the best approach. There is no need to have all the pathspecs
    # inside the combined event, however which one should be chosen is
    # perhaps something that can be evaluated here (regular TSK in favor of
    # an event stored deep inside a VSS for instance).

    for attribute_name in self._JOIN_ATTRIBUTES:
      first_value = getattr(first_event, attribute_name, None)
      if first_value is None:
        first_value_set = set()

      else:
        if isinstance(first_value, py2to3.STRING_TYPES):
          first_value = first_value.split(u';')
        else:
          first_value = [first_value]

        first_value_set = set(first_value)

      second_value = getattr(second_event, attribute_name, None)
      if second_value is None:
        second_value_set = set()

      else:
        if isinstance(second_value, py2to3.STRING_TYPES):
          second_value = second_value.split(u';')
        else:
          second_value = [second_value]

        second_value_set = set(second_value)

      values_list = list(first_value_set.union(second_value_set))
      values_list.sort()

      if not values_list:
        join_value = None
      elif len(values_list) == 1:
        join_value = values_list[0]
      else:
        join_value = u';'.join(values_list)

      setattr(first_event, attribute_name, join_value)

    # If two events are merged then we'll just pick the first inode value.
    inode = first_event.inode
    if isinstance(inode, py2to3.STRING_TYPES):
      inode_list = inode.split(u';')
      try:
        new_inode = int(inode_list[0], 10)
      except (IndexError, ValueError):
        new_inode = 0

      first_event.inode = new_inode

    # Special instance if this is a filestat entry we need to combine the
    # timestamp description field.
    parser_name = getattr(first_event, u'parser', None)

    if parser_name == u'filestat':
      first_description = getattr(first_event, u'timestamp_desc', u'')
      first_description_set = set(first_description.split(u';'))

      second_description = getattr(second_event, u'timestamp_desc', u'')
      second_description_set = set(second_description.split(u';'))

      if first_description_set.difference(second_description_set):
        descriptions_list = list(first_description_set.union(
            second_description_set))
        descriptions_list.sort()
        description_value = u';'.join(descriptions_list)

        setattr(first_event, u'timestamp_desc', description_value)



# -*- coding: utf-8 -*-
"""The extraction CLI tool."""

import yara

from plaso.cli import status_view_tool
from plaso.lib import definitions
from plaso.lib import errors
from plaso.lib import py2to3


class ExtractionTool(status_view_tool.StatusViewTool):
  """Class that implements an extraction CLI tool.

  Attributes:
    list_hashers (bool): True if the hashers should be listed.
    list_parsers_and_plugins (bool): True if the parsers and plugins should
        be listed.
  """

  # Approximately 250 MB of queued items per worker.
  _DEFAULT_QUEUE_SIZE = 125000

  # Enable the SHA256 hasher by default.
  _DEFAULT_HASHER_STRING = u'sha256'

  _BYTES_IN_A_MIB = 1024 * 1024

  def __init__(self, input_reader=None, output_writer=None):
    """Initializes the CLI tool object.

    Args:
      input_reader (Optional[InputReader]): input reader, where None indicates
          that the stdin input reader should be used.
      output_writer (Optional[OutputWriter]): output writer, where None
          indicates that the stdout output writer should be used.
    """
    super(ExtractionTool, self).__init__(
        input_reader=input_reader, output_writer=output_writer)
    self._buffer_size = 0
    self._filter_object = None
    self._force_preprocessing = False
    self._hasher_names_string = None
    self._mount_path = None
    self._operating_system = None
    self._output_module = None
    self._parser_filter_expression = None
    self._preferred_year = None
    self._process_archives = False
    self._process_compressed_streams = True
    self._queue_size = self._DEFAULT_QUEUE_SIZE
    self._single_process_mode = False
    self._storage_serializer_format = definitions.SERIALIZER_FORMAT_JSON
    self._text_prepend = None
    self._yara_rules_string = None

    self.list_hashers = False
    self.list_parsers_and_plugins = False

  def _ParseExtractionOptions(self, options):
    """Parses the extraction options.

    Args:
      options (argparse.Namespace): command line arguments.

    Raises:
      BadConfigOption: if the options are invalid.
    """
    self._hasher_names_string = getattr(
        options, u'hashers', self._DEFAULT_HASHER_STRING)
    if isinstance(self._hasher_names_string, py2to3.STRING_TYPES):
      if self._hasher_names_string.lower() == u'list':
        self.list_hashers = True

    yara_rules_path = getattr(options, u'yara_rules_path', None)
    if yara_rules_path:
      try:
        with open(yara_rules_path, 'rb') as rules_file:
          yara_rules_string = rules_file.read()
        # We try to parse the rules here, to check that the definitions are
        # valid. We then pass the string definitions along to the workers, so
        # that they don't need read access to the rules file.
        yara.compile(source=yara_rules_string)
        self._yara_rules_string = yara_rules_string
      except IOError as exception:
        raise errors.BadConfigObject(
            u'Unable to read Yara rules file: {0:s}, error was: {1!s}'.format(
                yara_rules_path, exception))
      except yara.Error as exception:
        raise errors.BadConfigObject(
            u'Unable to parse Yara rules in: {0:s}, error was: {1!s}'.format(
                yara_rules_path, exception))

    parser_filter_expression = self.ParseStringOption(
        options, u'parsers', default_value=u'')
    self._parser_filter_expression = parser_filter_expression.replace(
        u'\\', u'/')

    if (isinstance(self._parser_filter_expression, py2to3.STRING_TYPES) and
        self._parser_filter_expression.lower() == u'list'):
      self.list_parsers_and_plugins = True

    self._force_preprocessing = getattr(options, u'preprocess', False)

    self._preferred_year = getattr(options, u'preferred_year', None)
    if self._preferred_year:
      try:
        self._preferred_year = int(self._preferred_year, 10)
      except ValueError:
        raise errors.BadConfigOption(
            u'Invalid preferred year: {0:s}.'.format(self._preferred_year))

    self._process_archives = getattr(options, u'process_archives', False)
    self._process_compressed_streams = getattr(
        options, u'process_compressed_streams', True)

  def _ParsePerformanceOptions(self, options):
    """Parses the performance options.

    Args:
      options (argparse.Namespace): command line arguments.

    Raises:
      BadConfigOption: if the options are invalid.
    """
    self._buffer_size = getattr(options, u'buffer_size', 0)
    if self._buffer_size:
      # TODO: turn this into a generic function that supports more size
      # suffixes both MB and MiB and also that does not allow m as a valid
      # indicator for MiB since m represents milli not Mega.
      try:
        if self._buffer_size[-1].lower() == u'm':
          self._buffer_size = int(self._buffer_size[:-1], 10)
          self._buffer_size *= self._BYTES_IN_A_MIB
        else:
          self._buffer_size = int(self._buffer_size, 10)
      except ValueError:
        raise errors.BadConfigOption(
            u'Invalid buffer size: {0:s}.'.format(self._buffer_size))

    queue_size = getattr(options, u'queue_size', None)
    if queue_size:
      try:
        self._queue_size = int(queue_size, 10)
      except ValueError:
        raise errors.BadConfigOption(
            u'Invalid queue size: {0:s}.'.format(queue_size))

  def _ParseStorageOptions(self, options):
    """Parses the storage options.

    Args:
      options (argparse.Namespace): command line arguments.

    Raises:
      BadConfigOption: if the options are invalid.
    """
    serializer_format = getattr(
        options, u'serializer_format', definitions.SERIALIZER_FORMAT_JSON)
    if serializer_format not in definitions.SERIALIZER_FORMATS:
      raise errors.BadConfigOption(
          u'Unsupported storage serializer format: {0:s}.'.format(
              serializer_format))
    self._storage_serializer_format = serializer_format

  def AddExtractionOptions(self, argument_group):
    """Adds the extraction options to the argument group.

    Args:
      argument_group (argparse._ArgumentGroup): argparse argument group.
    """
    argument_group.add_argument(
        u'--hashers', dest=u'hashers', type=str, action=u'store',
        default=self._DEFAULT_HASHER_STRING, metavar=u'HASHER_LIST', help=(
            u'Define a list of hashers to use by the tool. This is a comma '
            u'separated list where each entry is the name of a hasher, such as '
            u'"md5,sha256". "all" indicates that all hashers should be '
            u'enabled. "none" disables all hashers. Use "--hashers list" or '
            u'"--info" to list the available hashers.'))

    argument_group.add_argument(
        u'--yara_rules', u'--yara-rules', dest=u'yara_rules_path',
        type=str, metavar=u'PATH', action=u'store', help=(
            u'Path to a file containing Yara rules definitions.'))

    # TODO: rename option name to parser_filter_expression.
    argument_group.add_argument(
        u'--parsers', dest=u'parsers', type=str, action=u'store',
        default=u'', metavar=u'PARSER_LIST', help=(
            u'Define a list of parsers to use by the tool. This is a comma '
            u'separated list where each entry can be either a name of a parser '
            u'or a parser list. Each entry can be prepended with an '
            u'exclamation mark to negate the selection (exclude it). The list '
            u'match is an exact match while an individual parser matching is '
            u'a case insensitive substring match, with support for glob '
            u'patterns. Examples would be: "reg" that matches the substring '
            u'"reg" in all parser names or the glob pattern "sky[pd]" that '
            u'would match all parsers that have the string "skyp" or "skyd" '
            u'in its name. All matching is case insensitive. Use "--parsers '
            u'list" or "--info" to list the available parsers.'))

    argument_group.add_argument(
        u'--preferred_year', u'--preferred-year', dest=u'preferred_year',
        action=u'store', default=None, metavar=u'YEAR', help=(
            u'When a format\'s timestamp does not include a year, e.g. '
            u'syslog, use this as the initial year instead of attempting '
            u'auto-detection.'))

    argument_group.add_argument(
        u'-p', u'--preprocess', dest=u'preprocess', action=u'store_true',
        default=False, help=(
            u'Turn on preprocessing. Preprocessing is turned on by default '
            u'when parsing image files, however if a mount point is being '
            u'parsed then this parameter needs to be set manually.'))

    argument_group.add_argument(
        u'--process_archives', u'--process-archives', dest=u'process_archives',
        action=u'store_true', default=False, help=(
            u'Process file entries embedded within archive files, such as '
            u'archive.tar and archive.zip. This can make processing '
            u'significantly slower.'))

    argument_group.add_argument(
        u'--skip_compressed_streams', u'--skip-compressed-streams',
        dest=u'process_compressed_streams', action=u'store_false', default=True,
        help=(
            u'Skip processing file content within compressed streams, such as '
            u'syslog.gz and syslog.bz2.'))

  def AddPerformanceOptions(self, argument_group):
    """Adds the performance options to the argument group.

    Args:
      argument_group (argparse._ArgumentGroup): argparse argument group.
    """
    argument_group.add_argument(
        u'--buffer_size', u'--buffer-size', u'--bs', dest=u'buffer_size',
        action=u'store', default=0, help=(
            u'The buffer size for the output (defaults to 196MiB).'))

    argument_group.add_argument(
        u'--queue_size', u'--queue-size', dest=u'queue_size', action=u'store',
        default=0, help=(
            u'The maximum number of queued items per worker '
            u'(defaults to {0:d})').format(self._DEFAULT_QUEUE_SIZE))

  def ParseOptions(self, options):
    """Parses tool specific options.

    Args:
      options (argparse.Namespace): command line arguments.

    Raises:
      BadConfigOption: if the options are invalid.
    """
    super(ExtractionTool, self).ParseOptions(options)
    self._ParseDataLocationOption(options)
    self._ParseFilterOptions(options)
    self._ParsePerformanceOptions(options)
    self._ParseStorageOptions(options)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the LastShutdown value plugin."""

import unittest

from plaso.formatters import shutdown  # pylint: disable=unused-import
from plaso.lib import eventdata
from plaso.lib import timelib
from plaso.parsers.winreg_plugins import shutdown

from tests import test_lib as shared_test_lib
from tests.parsers.winreg_plugins import test_lib


__author__ = 'Preston Miller, dpmforensics.com, github.com/prmiller91'


class ShutdownPluginTest(test_lib.RegistryPluginTestCase):
  """Tests for the LastShutdown value plugin."""

  @shared_test_lib.skipUnlessHasTestFile([u'SYSTEM'])
  def testProcess(self):
    """Tests the Process function."""
    test_file_entry = self._GetTestFileEntry([u'SYSTEM'])
    key_path = u'HKEY_LOCAL_MACHINE\\System\\ControlSet001\\Control\\Windows'

    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
    registry_key = win_registry.GetKeyByPath(key_path)

    plugin_object = shutdown.ShutdownPlugin()
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin_object, file_entry=test_file_entry)

    self.assertEqual(len(storage_writer.events), 1)

    event_object = storage_writer.events[0]

    self.assertEqual(event_object.pathspec, test_file_entry.path_spec)
    # This should just be the plugin name, as we're invoking it directly,
    # and not through the parser.
    self.assertEqual(event_object.parser, plugin_object.plugin_name)

    self.assertEqual(event_object.value_name, u'ShutdownTime')

    # Match UTC timestamp.
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2012-04-04 01:58:40.839249')
    self.assertEqual(event_object.timestamp, expected_timestamp)
    self.assertEqual(
        event_object.timestamp_desc, eventdata.EventTimestamp.LAST_SHUTDOWN)

    expected_message = (
        u'[{0:s}] '
        u'Description: ShutdownTime').format(key_path)
    expected_short_message = u'ShutdownTime'

    self._TestGetMessageStrings(
        event_object, expected_message, expected_short_message)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The Windows Prefetch event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager
from plaso.lib import errors


class WinPrefetchExecutionFormatter(interface.ConditionalEventFormatter):
  """Formatter for a Windows Prefetch execution event."""

  DATA_TYPE = u'windows:prefetch:execution'

  FORMAT_STRING_PIECES = [
      u'Prefetch',
      u'[{executable}] was executed -',
      u'run count {run_count}',
      u'path: {path}',
      u'hash: 0x{prefetch_hash:08X}',
      u'{volumes_string}']

  FORMAT_STRING_SHORT_PIECES = [
      u'{executable} was run',
      u'{run_count} time(s)']

  SOURCE_LONG = u'WinPrefetch'
  SOURCE_SHORT = u'LOG'

  def GetMessages(self, unused_formatter_mediator, event):
    """Determines the formatted message strings for an event object.

    Args:
      formatter_mediator (FormatterMediator): mediates the interactions between
          formatters and other components, such as storage and Windows EventLog
          resources.
      event (EventObject): event.

    Returns:
      tuple(str, str): formatted message string and short message string.

    Raises:
      WrongFormatter: if the event object cannot be formatted by the formatter.
    """
    if self.DATA_TYPE != event.data_type:
      raise errors.WrongFormatter(
          u'Invalid event object - unsupported data type: {0:s}'.format(
              event.data_type))

    event_values = event.CopyToDict()

    number_of_volumes = event_values.get(u'number_of_volumes', 0)
    volume_serial_numbers = event_values.get(u'volume_serial_numbers', None)
    volume_device_paths = event_values.get(u'volume_device_paths', None)
    volumes_strings = []
    for volume_index in range(0, number_of_volumes):
      if not volume_serial_numbers:
        volume_serial_number = u'UNKNOWN'
      else:
        volume_serial_number = volume_serial_numbers[volume_index]

      if not volume_device_paths:
        volume_device_path = u'UNKNOWN'
      else:
        volume_device_path = volume_device_paths[volume_index]

      volumes_strings.append((
          u'volume: {0:d} [serial number: 0x{1:08X}, device path: '
          u'{2:s}]').format(
              volume_index + 1, volume_serial_number, volume_device_path))

    if volumes_strings:
      event_values[u'volumes_string'] = u', '.join(volumes_strings)

    return self._ConditionalFormatMessages(event_values)


manager.FormattersManager.RegisterFormatter(WinPrefetchExecutionFormatter)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the MSIE typed URLs Windows Registry plugin."""

import unittest

from plaso.formatters import winreg  # pylint: disable=unused-import
from plaso.lib import timelib
from plaso.parsers.winreg_plugins import typedurls

from tests import test_lib as shared_test_lib
from tests.parsers.winreg_plugins import test_lib


__author__ = 'David Nides (david.nides@gmail.com)'


class MsieTypedURLsPluginTest(test_lib.RegistryPluginTestCase):
  """Tests for the MSIE typed URLs Windows Registry plugin."""

  @shared_test_lib.skipUnlessHasTestFile([u'NTUSER-WIN7.DAT'])
  def testProcess(self):
    """Tests the Process function."""
    test_file_entry = self._GetTestFileEntry([u'NTUSER-WIN7.DAT'])
    key_path = (
        u'HKEY_CURRENT_USER\\Software\\Microsoft\\Internet Explorer\\'
        u'TypedURLs')

    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
    registry_key = win_registry.GetKeyByPath(key_path)

    plugin_object = typedurls.TypedURLsPlugin()
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin_object, file_entry=test_file_entry)

    self.assertEqual(len(storage_writer.events), 1)

    event_object = storage_writer.events[0]

    self.assertEqual(event_object.pathspec, test_file_entry.path_spec)
    # This should just be the plugin name, as we're invoking it directly,
    # and not through the parser.
    self.assertEqual(event_object.parser, plugin_object.plugin_name)

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2012-03-12 21:23:53.307749')
    self.assertEqual(event_object.timestamp, expected_timestamp)

    regvalue_identifier = u'url1'
    expected_value = u'http://cnn.com/'
    self._TestRegvalue(event_object, regvalue_identifier, expected_value)

    expected_message = (
        u'[{0:s}] '
        u'url1: http://cnn.com/ '
        u'url10: http://www.adobe.com/ '
        u'url11: http://www.google.com/ '
        u'url12: http://www.firefox.com/ '
        u'url13: http://go.microsoft.com/fwlink/?LinkId=69157 '
        u'url2: http://twitter.com/ '
        u'url3: http://linkedin.com/ '
        u'url4: http://tweetdeck.com/ '
        u'url5: mozilla '
        u'url6: http://google.com/ '
        u'url7: http://controller.shieldbase.local/certsrv/ '
        u'url8: http://controller.shieldbase.local/ '
        u'url9: http://www.stark-research-labs.com/').format(key_path)
    expected_short_message = u'{0:s}...'.format(expected_message[:77])

    self._TestGetMessageStrings(
        event_object, expected_message, expected_short_message)


class TypedPathsPluginTest(test_lib.RegistryPluginTestCase):
  """Tests for the typed paths Windows Registry plugin."""

  @shared_test_lib.skipUnlessHasTestFile([u'NTUSER-WIN7.DAT'])
  def testProcess(self):
    """Tests the Process function."""
    test_file_entry = self._GetTestFileEntry([u'NTUSER-WIN7.DAT'])
    key_path = (
        u'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
        u'Explorer\\TypedPaths')

    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
    registry_key = win_registry.GetKeyByPath(key_path)

    plugin_object = typedurls.TypedURLsPlugin()
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin_object, file_entry=test_file_entry)

    self.assertEqual(len(storage_writer.events), 1)

    event_object = storage_writer.events[0]

    self.assertEqual(event_object.pathspec, test_file_entry.path_spec)
    # This should just be the plugin name, as we're invoking it directly,
    # and not through the parser.
    self.assertEqual(event_object.parser, plugin_object.plugin_name)

    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2010-11-10 07:58:15.811625')
    self.assertEqual(event_object.timestamp, expected_timestamp)

    regvalue_identifier = u'url1'
    expected_value = u'\\\\controller'
    self._TestRegvalue(event_object, regvalue_identifier, expected_value)

    expected_message = u'[{0:s}] {1:s}: {2:s}'.format(
        key_path, regvalue_identifier, expected_value)
    expected_short_message = u'{0:s}...'.format(expected_message[:77])
    self._TestGetMessageStrings(
        event_object, expected_message, expected_short_message)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""This file contains the Spotlight Volume Configuration plist in Plaso."""

from plaso.containers import plist_event
from plaso.parsers import plist
from plaso.parsers.plist_plugins import interface


__author__ = 'Joaquin Moreno Garijo (Joaquin.MorenoGarijo.2013@live.rhul.ac.uk)'


class SpotlightVolumePlugin(interface.PlistPlugin):
  """Basic plugin to extract the Spotlight Volume Configuration."""

  NAME = u'spotlight_volume'
  DESCRIPTION = u'Parser for Spotlight volume configuration plist files.'

  PLIST_PATH = u'VolumeConfiguration.plist'
  PLIST_KEYS = frozenset([u'Stores'])

  def GetEntries(self, parser_mediator, match=None, **unused_kwargs):
    """Extracts relevant VolumeConfiguration Spotlight entries.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      match (Optional[dict[str: object]]): keys extracted from PLIST_KEYS.
    """
    for volume_name, volume in iter(match[u'Stores'].items()):
      description = u'Spotlight Volume {0:s} ({1:s}) activated.'.format(
          volume_name, volume[u'PartialPath'])
      event_object = plist_event.PlistEvent(
          u'/Stores', u'', volume[u'CreationDate'], description)
      parser_mediator.ProduceEvent(event_object)


plist.PlistParser.RegisterPlugin(SpotlightVolumePlugin)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the browser search analysis plugin."""

import unittest

from plaso.analysis import browser_search
from plaso.parsers import sqlite

from tests import test_lib as shared_test_lib
from tests.analysis import test_lib


@shared_test_lib.skipUnlessHasTestFile([u'History'])
class BrowserSearchAnalysisTest(test_lib.AnalysisPluginTestCase):
  """Tests for the browser search analysis plugin."""

  def testExamineEventAndCompileReport(self):
    """Tests the ExamineEvent and CompileReport functions."""
    parser = sqlite.SQLiteParser()
    plugin = browser_search.BrowserSearchPlugin()

    storage_writer = self._ParseAndAnalyzeFile([u'History'], parser, plugin)

    self.assertEqual(len(storage_writer.events), 71)

    self.assertEqual(len(storage_writer.analysis_reports), 1)

    analysis_report = storage_writer.analysis_reports[0]

    # Due to the behavior of the join one additional empty string at the end
    # is needed to create the last empty line.
    expected_text = u'\n'.join([
        u' == ENGINE: Google Search ==',
        u'1 really really funny cats',
        u'1 java plugin',
        u'1 funnycats.exe',
        u'1 funny cats',
        u'',
        u''])

    self.assertEqual(analysis_report.text, expected_text)
    self.assertEqual(analysis_report.plugin_name, u'browser_search')

    expected_keys = set([u'Google Search'])
    self.assertEqual(set(analysis_report.report_dict.keys()), expected_keys)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Parser for syslog formatted log files"""
import re

import pyparsing

from plaso.containers import events
from plaso.containers import time_events
from plaso.lib import errors
from plaso.lib import eventdata
from plaso.lib import timelib
from plaso.parsers import manager
from plaso.parsers import text_parser


class SyslogLineEventData(events.EventData):
  """Syslog line event data.

  Attributes:
    body (str): message body.
    hostname (str): hostname of the reporter.
    pid (str): process identifier of the reporter.
    reporter (str): reporter.
    severity (str): severity.
  """

  DATA_TYPE = u'syslog:line'

  def __init__(self, data_type=DATA_TYPE):
    """Initializes an event data attribute container.

    Args:
      data_type (Optional[str]): event data type indicator.
    """
    super(SyslogLineEventData, self).__init__(data_type=data_type)
    self.body = None
    self.hostname = None
    self.pid = None
    self.reporter = None
    self.severity = None


class SyslogCommentEventData(events.EventData):
  """Syslog comment event data.

  Attributes:
    body (str): message body.
  """

  DATA_TYPE = u'syslog:comment'

  def __init__(self):
    """Initializes event data."""
    super(SyslogCommentEventData, self).__init__(data_type=self.DATA_TYPE)
    self.body = None


class SyslogParser(text_parser.PyparsingMultiLineTextParser):
  """Parses syslog formatted log files"""
  NAME = u'syslog'

  DESCRIPTION = u'Syslog Parser'

  _ENCODING = u'utf-8'

  _plugin_classes = {}

  # The reporter and facility fields can contain any printable character, but
  # to allow for processing of syslog formats that delimit the reporter and
  # facility with printable characters, we remove certain common delimiters
  # from the set of printable characters.
  _REPORTER_CHARACTERS = u''.join(
      [c for c in pyparsing.printables if c not in [u':', u'[', u'<']])
  _FACILITY_CHARACTERS = u''.join(
      [c for c in pyparsing.printables if c not in [u':', u'>']])

  _SYSLOG_SEVERITY = [
      u'EMERG',
      u'ALERT',
      u'CRIT',
      u'ERR',
      u'WARNING',
      u'NOTICE',
      u'INFO',
      u'DEBUG']

  _OFFSET_PREFIX = [
      u'-',
      u'+']

  _BODY_CONTENT = (r'.*?(?=($|\n\w{3}\s+\d{1,2}\s\d{2}:\d{2}:\d{2})|' \
                   r'($|\n\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\.\d{6}' \
                   r'[\+|-]\d{2}:\d{2}\s))')

  _VERIFICATION_REGEX = \
      re.compile(r'^\w{3}\s+\d{1,2}\s\d{2}:\d{2}:\d{2}\s' +
                 _BODY_CONTENT)

  # The Chrome OS syslog messages are of a format begininng with an
  # ISO 8601 combined date and time expression with timezone designator:
  #   2016-10-25T12:37:23.297265-07:00
  #
  # This will then be followed by the SYSLOG Severity which will be one of:
  #   EMERG,ALERT,CRIT,ERR,WARNING,NOTICE,INFO,DEBUG
  #
  # 2016-10-25T12:37:23.297265-07:00 INFO
  _CHROMEOS_VERIFICATION_REGEX = \
      re.compile(r'^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\.'
                 r'\d{6}[\+|-]\d{2}:\d{2}\s'
                 r'(EMERG|ALERT|CRIT|ERR|WARNING|NOTICE|INFO|DEBUG)' +
                 _BODY_CONTENT)

  _PYPARSING_COMPONENTS = {
      u'year': text_parser.PyparsingConstants.FOUR_DIGITS.setResultsName(
          u'year'),
      u'two_digit_month': (
          text_parser.PyparsingConstants.TWO_DIGITS.setResultsName(
              u'two_digit_month')),
      u'month': text_parser.PyparsingConstants.MONTH.setResultsName(u'month'),
      u'day': text_parser.PyparsingConstants.ONE_OR_TWO_DIGITS.setResultsName(
          u'day'),
      u'hour': text_parser.PyparsingConstants.TWO_DIGITS.setResultsName(
          u'hour'),
      u'minute': text_parser.PyparsingConstants.TWO_DIGITS.setResultsName(
          u'minute'),
      u'second': text_parser.PyparsingConstants.TWO_DIGITS.setResultsName(
          u'second'),
      u'fractional_seconds': pyparsing.Word(pyparsing.nums).setResultsName(
          u'fractional_seconds'),
      u'hostname': pyparsing.Word(pyparsing.printables).setResultsName(
          u'hostname'),
      u'reporter': pyparsing.Word(_REPORTER_CHARACTERS).setResultsName(
          u'reporter'),
      u'pid': text_parser.PyparsingConstants.PID.setResultsName(u'pid'),
      u'facility': pyparsing.Word(_FACILITY_CHARACTERS).setResultsName(
          u'facility'),
      u'severity': pyparsing.oneOf(_SYSLOG_SEVERITY).setResultsName(
          u'severity'),
      u'body': pyparsing.Regex(_BODY_CONTENT, re.DOTALL).setResultsName(
          u'body'),
      u'comment_body': pyparsing.SkipTo(u' ---').setResultsName(
          u'body'),
      u'iso_8601_offset': (
          pyparsing.oneOf(_OFFSET_PREFIX) +
          text_parser.PyparsingConstants.TWO_DIGITS +
          pyparsing.Optional(
              pyparsing.Literal(u':') +
              text_parser.PyparsingConstants.TWO_DIGITS))
  }

  _PYPARSING_COMPONENTS[u'date'] = (
      _PYPARSING_COMPONENTS[u'month'] +
      _PYPARSING_COMPONENTS[u'day'] +
      _PYPARSING_COMPONENTS[u'hour'] + pyparsing.Suppress(u':') +
      _PYPARSING_COMPONENTS[u'minute'] + pyparsing.Suppress(u':') +
      _PYPARSING_COMPONENTS[u'second'] + pyparsing.Optional(
          pyparsing.Suppress(u'.') +
          _PYPARSING_COMPONENTS[u'fractional_seconds']))

  _PYPARSING_COMPONENTS[u'iso_8601_date'] = pyparsing.Combine(
      _PYPARSING_COMPONENTS[u'year'] + pyparsing.Literal(u'-') +
      _PYPARSING_COMPONENTS[u'two_digit_month'] + pyparsing.Literal(u'-') +
      _PYPARSING_COMPONENTS[u'day'] + pyparsing.Literal(u'T') +
      _PYPARSING_COMPONENTS[u'hour'] + pyparsing.Literal(u':') +
      _PYPARSING_COMPONENTS[u'minute'] + pyparsing.Literal(u':') +
      _PYPARSING_COMPONENTS[u'second'] + pyparsing.Literal(u'.') +
      _PYPARSING_COMPONENTS[u'fractional_seconds'] +
      _PYPARSING_COMPONENTS[u'iso_8601_offset'],
      joinString=u'', adjacent=True).setResultsName(u'iso_8601_date')

  _CHROMEOS_SYSLOG_LINE = (
      _PYPARSING_COMPONENTS[u'iso_8601_date'] +
      _PYPARSING_COMPONENTS[u'severity'] +
      _PYPARSING_COMPONENTS[u'reporter'] +
      pyparsing.Optional(pyparsing.Suppress(u':')) +
      pyparsing.Optional(
          pyparsing.Suppress(u'[') + _PYPARSING_COMPONENTS[u'pid'] +
          pyparsing.Suppress(u']')) +
      pyparsing.Optional(pyparsing.Suppress(u':')) +
      _PYPARSING_COMPONENTS[u'body'] + pyparsing.lineEnd())

  _SYSLOG_LINE = (
      _PYPARSING_COMPONENTS[u'date'] +
      _PYPARSING_COMPONENTS[u'hostname'] +
      _PYPARSING_COMPONENTS[u'reporter'] +
      pyparsing.Optional(
          pyparsing.Suppress(u'[') + _PYPARSING_COMPONENTS[u'pid'] +
          pyparsing.Suppress(u']')) +
      pyparsing.Optional(
          pyparsing.Suppress(u'<') + _PYPARSING_COMPONENTS[u'facility'] +
          pyparsing.Suppress(u'>')) +
      pyparsing.Optional(pyparsing.Suppress(u':')) +
      _PYPARSING_COMPONENTS[u'body'] + pyparsing.lineEnd())

  _SYSLOG_COMMENT = (
      _PYPARSING_COMPONENTS[u'date'] + pyparsing.Suppress(u':') +
      pyparsing.Suppress(u'---') + _PYPARSING_COMPONENTS[u'comment_body'] +
      pyparsing.Suppress(u'---') + pyparsing.LineEnd())

  _KERNEL_SYSLOG_LINE = (
      _PYPARSING_COMPONENTS[u'date'] +
      pyparsing.Literal(u'kernel').setResultsName(u'reporter') +
      pyparsing.Suppress(u':') + _PYPARSING_COMPONENTS[u'body'] +
      pyparsing.lineEnd())

  LINE_STRUCTURES = [
      (u'syslog_line', _SYSLOG_LINE),
      (u'syslog_line', _KERNEL_SYSLOG_LINE),
      (u'syslog_comment', _SYSLOG_COMMENT),
      (u'chromeos_syslog_line', _CHROMEOS_SYSLOG_LINE)]

  _SUPPORTED_KEYS = frozenset([key for key, _ in LINE_STRUCTURES])

  def __init__(self):
    """Initializes a parser object."""
    super(SyslogParser, self).__init__()
    self._last_month = 0
    self._maximum_year = 0
    self._plugin_objects_by_reporter = {}
    self._year_use = 0

  def _UpdateYear(self, mediator, month):
    """Updates the year to use for events, based on last observed month.

    Args:
      mediator (ParserMediator): mediates the interactions between
          parsers and other components, such as storage and abort signals.
      month (int): month observed by the parser, where January is 1.
    """
    if not self._year_use:
      self._year_use = mediator.GetEstimatedYear()
    if not self._maximum_year:
      self._maximum_year = mediator.GetLatestYear()

    if not self._last_month:
      self._last_month = month
      return

    # Some syslog daemons allow out-of-order sequences, so allow some leeway
    # to not cause Apr->May->Apr to cause the year to increment.
    # See http://bugzilla.adiscon.com/show_bug.cgi?id=527
    if self._last_month > (month + 1):
      if self._year_use != self._maximum_year:
        self._year_use += 1
    self._last_month = month

  def EnablePlugins(self, plugin_includes):
    """Enables parser plugins.

    Args:
      plugin_includes (list[str]): names of the plugins to enable, where None
          or an empty list represents all plugins. Note that the default plugin
          is handled separately.
    """
    super(SyslogParser, self).EnablePlugins(plugin_includes)

    self._plugin_objects_by_reporter = {}
    for plugin_object in self._plugin_objects:
      self._plugin_objects_by_reporter[plugin_object.REPORTER] = plugin_object

  def ParseRecord(self, mediator, key, structure):
    """Parses a matching entry.

    Args:
      mediator (ParserMediator): mediates the interactions between
          parsers and other components, such as storage and abort signals.
      key (str): name of the parsed structure.
      structure (pyparsing.ParseResults): elements parsed from the file.

    Raises:
      ParseError: when the structure type is unknown.
    """
    if key not in self._SUPPORTED_KEYS:
      raise errors.ParseError(
          u'Unable to parse record, unknown structure: {0:s}'.format(key))

    if key == u'chromeos_syslog_line':
      timestamp = timelib.Timestamp.FromTimeString(structure.iso_8601_date[0])
    else:
      month = timelib.MONTH_DICT.get(structure.month.lower(), None)
      if not month:
        mediator.ProduceParserError(u'Invalid month value: {0:s}'.format(month))
        return

      self._UpdateYear(mediator, month)
      timestamp = timelib.Timestamp.FromTimeParts(
          year=self._year_use, month=month, day=structure.day,
          hour=structure.hour, minutes=structure.minute,
          seconds=structure.second, timezone=mediator.timezone)

    plugin_object = None
    if key == u'syslog_comment':
      event_data = SyslogCommentEventData()
      event_data.body = structure.body
      # TODO: pass line number to offset or remove.
      event_data.offset = 0

    else:
      event_data = SyslogLineEventData()
      event_data.body = structure.body
      event_data.hostname = structure.hostname or None
      # TODO: pass line number to offset or remove.
      event_data.offset = 0
      event_data.pid = structure.pid
      event_data.reporter = structure.reporter
      event_data.severity = structure.severity

      plugin_object = self._plugin_objects_by_reporter.get(
          structure.reporter, None)
      if plugin_object:
        attributes = {
            u'hostname': structure.hostname,
            u'severity': structure.severity,
            u'reporter': structure.reporter,
            u'pid': structure.pid,
            u'body': structure.body}

        try:
          # TODO: pass event_data instead of attributes.
          plugin_object.Process(mediator, timestamp, attributes)

        except errors.WrongPlugin:
          plugin_object = None

    if not plugin_object:
      event = time_events.TimestampEvent(
          timestamp, eventdata.EventTimestamp.WRITTEN_TIME)
      mediator.ProduceEventWithEventData(event, event_data)

  def VerifyStructure(self, unused_mediator, line):
    """Verifies that this is a syslog-formatted file.

    Args:
      mediator (ParserMediator): mediates the interactions between
          parsers and other components, such as storage and abort signals.
      line (str): single line from the text file.

    Returns:
      bool: whether the line appears to contain syslog content.
    """
    return (re.match(self._VERIFICATION_REGEX, line) or
            re.match(self._CHROMEOS_VERIFICATION_REGEX, line)) is not None


manager.ParsersManager.RegisterParser(SyslogParser)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Popularity Contest (popcontest) parser."""

import unittest

from plaso.formatters import popcontest  # pylint: disable=unused-import
from plaso.lib import eventdata
from plaso.lib import timelib
from plaso.parsers import popcontest

from tests import test_lib as shared_test_lib
from tests.parsers import test_lib


__author__ = 'Francesco Picasso (francesco.picasso@gmail.com)'


class PopularityContestUnitTest(test_lib.ParserTestCase):
  """Tests for the popcontest parser."""

  @shared_test_lib.skipUnlessHasTestFile([u'popcontest1.log'])
  def testParse(self):
    """Tests the Parse function."""
    parser_object = popcontest.PopularityContestParser()
    storage_writer = self._ParseFile([u'popcontest1.log'], parser_object)

    self.assertEqual(len(storage_writer.events), 22)

    event_object = storage_writer.events[0]

    self.assertEqual(
        event_object.timestamp_desc, eventdata.EventTimestamp.ADDED_TIME)
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2010-06-22 05:41:41')
    self.assertEqual(event_object.timestamp, expected_timestamp)

    expected_string = (
        u'Session 0 start '
        u'ID 12345678901234567890123456789012 [ARCH:i386 POPCONVER:1.38]')
    expected_short_string = u'Session 0 start'
    self._TestGetMessageStrings(
        event_object, expected_string, expected_short_string)

    event_object = storage_writer.events[1]

    self.assertEqual(
        event_object.timestamp_desc, eventdata.EventTimestamp.ACCESS_TIME)
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2010-06-22 07:34:42')
    self.assertEqual(event_object.timestamp, expected_timestamp)

    expected_string = u'mru [/usr/sbin/atd] package [at]'
    expected_short_string = u'/usr/sbin/atd'
    self._TestGetMessageStrings(
        event_object, expected_string, expected_short_string)

    event_object = storage_writer.events[3]

    self.assertEqual(
        event_object.timestamp_desc, eventdata.EventTimestamp.ACCESS_TIME)
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2010-06-22 07:34:43')
    self.assertEqual(event_object.timestamp, expected_timestamp)

    expected_string = (
        u'mru [/usr/lib/python2.5/lib-dynload/_struct.so] '
        u'package [python2.5-minimal]')
    expected_short_string = u'/usr/lib/python2.5/lib-dynload/_struct.so'
    self._TestGetMessageStrings(
        event_object, expected_string, expected_short_string)

    event_object = storage_writer.events[5]

    self.assertEqual(
        event_object.timestamp_desc, eventdata.EventTimestamp.ACCESS_TIME)
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2010-05-30 05:26:20')
    self.assertEqual(event_object.timestamp, expected_timestamp)

    expected_string = (
        u'mru [/usr/bin/empathy] package [empathy] tag [RECENT-CTIME]')
    expected_short_string = u'/usr/bin/empathy'
    self._TestGetMessageStrings(
        event_object, expected_string, expected_short_string)

    event_object = storage_writer.events[6]

    self.assertEqual(
        event_object.timestamp_desc,
        eventdata.EventTimestamp.ENTRY_MODIFICATION_TIME)
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2010-05-30 05:27:43')
    self.assertEqual(event_object.timestamp, expected_timestamp)

    expected_string = (
        u'mru [/usr/bin/empathy] package [empathy] tag [RECENT-CTIME]')
    expected_short_string = u'/usr/bin/empathy'
    self._TestGetMessageStrings(
        event_object, expected_string, expected_short_string)

    event_object = storage_writer.events[11]

    self.assertEqual(
        event_object.timestamp_desc, eventdata.EventTimestamp.ACCESS_TIME)
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2010-05-12 07:58:33')
    self.assertEqual(event_object.timestamp, expected_timestamp)

    expected_string = u'mru [/usr/bin/orca] package [gnome-orca] tag [OLD]'
    expected_short_string = u'/usr/bin/orca'
    self._TestGetMessageStrings(
        event_object, expected_string, expected_short_string)

    event_object = storage_writer.events[13]

    self.assertEqual(
        event_object.timestamp_desc, eventdata.EventTimestamp.ADDED_TIME)
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2010-06-22 05:41:41')
    self.assertEqual(event_object.timestamp, expected_timestamp)

    expected_string = u'Session 0 end'
    expected_short_string = expected_string
    self._TestGetMessageStrings(
        event_object, expected_string, expected_short_string)

    event_object = storage_writer.events[14]

    self.assertEqual(
        event_object.timestamp_desc, eventdata.EventTimestamp.ADDED_TIME)
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2010-06-22 05:41:41')
    self.assertEqual(event_object.timestamp, expected_timestamp)

    expected_string = (
        u'Session 1 start '
        u'ID 12345678901234567890123456789012 [ARCH:i386 POPCONVER:1.38]')
    expected_short_string = u'Session 1 start'
    self._TestGetMessageStrings(
        event_object, expected_string, expected_short_string)

    event_object = storage_writer.events[15]

    self.assertEqual(
        event_object.timestamp_desc, eventdata.EventTimestamp.ACCESS_TIME)
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2010-06-22 07:34:42')
    self.assertEqual(event_object.timestamp, expected_timestamp)

    expected_string = u'mru [/super/cool/plasuz] package [plaso]'
    expected_short_string = u'/super/cool/plasuz'
    self._TestGetMessageStrings(
        event_object, expected_string, expected_short_string)

    event_object = storage_writer.events[18]

    self.assertEqual(
        event_object.timestamp_desc, eventdata.EventTimestamp.ACCESS_TIME)
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2010-04-06 12:25:42')
    self.assertEqual(event_object.timestamp, expected_timestamp)

    expected_string = u'mru [/super/cool/plasuz] package [miss_ctime]'
    expected_short_string = u'/super/cool/plasuz'
    self._TestGetMessageStrings(
        event_object, expected_string, expected_short_string)

    event_object = storage_writer.events[19]

    self.assertEqual(
        event_object.timestamp_desc, eventdata.EventTimestamp.ACCESS_TIME)
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2010-05-12 07:58:33')
    self.assertEqual(event_object.timestamp, expected_timestamp)

    expected_string = u'mru [/super/cóól] package [plaso] tag [WRONG_TAG]'
    expected_short_string = u'/super/cóól'
    self._TestGetMessageStrings(
        event_object, expected_string, expected_short_string)

    event_object = storage_writer.events[21]

    self.assertEqual(
        event_object.timestamp_desc, eventdata.EventTimestamp.ADDED_TIME)
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2010-06-22 05:41:41')
    self.assertEqual(event_object.timestamp, expected_timestamp)

    expected_string = u'Session 1 end'
    expected_short_string = expected_string
    self._TestGetMessageStrings(
        event_object, expected_string, expected_short_string)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Analysis plugin to look up files in VirusTotal and tag events."""

import logging

from plaso.analysis import interface
from plaso.analysis import manager
from plaso.lib import errors


class VirusTotalAnalyzer(interface.HTTPHashAnalyzer):
  """Class that analyzes file hashes by consulting VirusTotal."""

  _VIRUSTOTAL_API_REPORT_URL = (
      u'https://www.virustotal.com/vtapi/v2/file/report')

  SUPPORTED_HASHES = [u'md5', u'sha1', u'sha256']

  def __init__(self, hash_queue, hash_analysis_queue, **kwargs):
    """Initializes a VirusTotal Analyzer thread.

    Args:
      hash_queue (Queue.queue): queue that contains hashes to be analyzed.
      hash_analysis_queue (Queue.queue): queue the analyzer will append
          HashAnalysis objects to.
    """
    super(VirusTotalAnalyzer, self).__init__(
        hash_queue, hash_analysis_queue, **kwargs)
    self._api_key = None
    self._checked_for_old_python_version = False

  def SetAPIKey(self, api_key):
    """Sets the VirusTotal API key to use in queries.

    Args:
      api_key (str): VirusTotal API key
    """
    self._api_key = api_key

  def Analyze(self, hashes):
    """Looks up hashes in VirusTotal using the VirusTotal HTTP API.

    The API is documented here:
      https://www.virustotal.com/en/documentation/public-api/

    Args:
      hashes (list[str]): hashes to look up.

    Returns:
      list[HashAnalysis]: analysis results.

    Raises:
      RuntimeError: If the VirusTotal API key has not been set.
    """
    if not self._api_key:
      raise RuntimeError(u'No API key specified for VirusTotal lookup.')

    hash_analyses = []
    resource_string = u', '.join(hashes)
    params = {u'apikey': self._api_key, u'resource': resource_string}
    try:
      json_response = self.MakeRequestAndDecodeJSON(
          self._VIRUSTOTAL_API_REPORT_URL, u'GET', params=params)
    except errors.ConnectionError as exception:
      logging.error(
          (u'Error communicating with VirusTotal {0:s}. VirusTotal plugin is '
           u'aborting.').format(exception))
      self.SignalAbort()
      return hash_analyses

    # The content of the response from VirusTotal has a different structure if
    # one or more than one hash is looked up at once.
    if isinstance(json_response, dict):
      # Only one result.
      resource = json_response[u'resource']
      hash_analysis = interface.HashAnalysis(resource, json_response)
      hash_analyses.append(hash_analysis)
    else:
      for result in json_response:
        resource = result[u'resource']
        hash_analysis = interface.HashAnalysis(resource, result)
        hash_analyses.append(hash_analysis)
    return hash_analyses


class VirusTotalAnalysisPlugin(interface.HashTaggingAnalysisPlugin):
  """An analysis plugin for looking up hashes in VirusTotal."""

  # TODO: Check if there are other file types worth checking VirusTotal for.
  DATA_TYPES = [u'pe:compilation:compilation_time']

  URLS = [u'https://virustotal.com']

  NAME = u'virustotal'

  _VIRUSTOTAL_NOT_PRESENT_RESPONSE_CODE = 0
  _VIRUSTOTAL_PRESENT_RESPONSE_CODE = 1
  _VIRUSTOTAL_ANALYSIS_PENDING_RESPONSE_CODE = -2

  def __init__(self):
    """Initializes a VirusTotal analysis plugin."""
    super(VirusTotalAnalysisPlugin, self).__init__(VirusTotalAnalyzer)
    self._api_key = None

  def EnableFreeAPIKeyRateLimit(self):
    """Configures Rate limiting for queries to VirusTotal.

    The default rate limit for free VirusTotal API keys is 4 requests per
    minute.
    """
    self._analyzer.hashes_per_batch = 4
    self._analyzer.wait_after_analysis = 60
    self._analysis_queue_timeout = self._analyzer.wait_after_analysis + 1

  def GenerateLabels(self, hash_information):
    """Generates a list of strings that will be used in the event tag.

    Args:
      hash_information (dict[str, object]): the JSON decoded contents of the
          result of a VirusTotal lookup, as produced by the VirusTotalAnalyzer.

    Returns:
      list[str]: strings describing the results from VirusTotal.
    """
    response_code = hash_information[u'response_code']
    if response_code == self._VIRUSTOTAL_NOT_PRESENT_RESPONSE_CODE:
      return [u'virustotal_not_present']
    elif response_code == self._VIRUSTOTAL_PRESENT_RESPONSE_CODE:
      positives = hash_information[u'positives']
      if positives > 0:
        return [u'virustotal_detections_{0:d}'.format(positives)]
      return [u'virsutotal_no_detections']
    elif response_code == self._VIRUSTOTAL_ANALYSIS_PENDING_RESPONSE_CODE:
      return [u'virustotal_analysis_pending']
    else:
      logging.error(
          u'VirusTotal returned unknown response code {0!s}'.format(
              response_code))
      return [u'virustotal_unknown_response_code_{0:d}'.format(response_code)]

  def SetAPIKey(self, api_key):
    """Sets the VirusTotal API key to use in queries.

    Args:
      api_key (str): VirusTotal API key
    """
    self._analyzer.SetAPIKey(api_key)


manager.AnalysisPluginManager.RegisterPlugin(VirusTotalAnalysisPlugin)



# -*- coding: utf-8 -*-
"""The StatusView CLI tool."""

import sys

try:
  import win32console
except ImportError:
  win32console = None

import plaso

from plaso.lib import definitions
from plaso.cli import storage_media_tool
from plaso.cli import tools as cli_tools
from plaso.cli import views as cli_views


class StatusViewTool(storage_media_tool.StorageMediaTool):
  """A tool that reports extraction status."""

  def __init__(self, input_reader=None, output_writer=None):
    """Initializes the status view tool object.

    Args:
      input_reader (Optional[InputReader]): input reader, where None indicates
          that the stdin input reader should be used.
      output_writer (Optional[OutputWriter]): output writer, where None
          indicates that the stdout output writer should be used.
    """
    super(StatusViewTool, self).__init__(
        input_reader=input_reader, output_writer=output_writer)

    self._stdout_output_writer = isinstance(
        self._output_writer, cli_tools.StdoutOutputWriter)

    self._number_of_analysis_reports = 0
    self._source_path = None
    self._source_type_string = u'UNKNOWN'

  def _PrintStatusHeader(self):
    """Prints the processing status header."""
    self._output_writer.Write(
        u'Source path\t: {0:s}\n'.format(self._source_path))
    self._output_writer.Write(
        u'Source type\t: {0:s}\n'.format(self._source_type_string))

    if self._filter_file:
      self._output_writer.Write(u'Filter file\t: {0:s}\n'.format(
          self._filter_file))

    self._output_writer.Write(u'\n')

  def _FormatSizeInUnitsOf1024(self, size):
    """Represents a number of bytes in units of 1024.

    Args:
      size (int): size in bytes.

    Returns:
      str: human readable string of the size.
    """
    magnitude_1024 = 0
    used_memory_1024 = float(size)
    while used_memory_1024 >= 1024:
      used_memory_1024 /= 1024
      magnitude_1024 += 1

    if magnitude_1024 > 0 and magnitude_1024 <= 7:
      return u'{0:.1f} {1:s}'.format(
          used_memory_1024, self._UNITS_1024[magnitude_1024])

    return u'{0:d} B'.format(size)

  def _FormatStatusTableRow(self, process_status):
    """Formats a status table row.

    Args:
      process_status (ProcessStatus): processing status.
    """
    # This check makes sure the columns are tab aligned.
    identifier = process_status.identifier
    if len(identifier) < 8:
      identifier = u'{0:s}\t'.format(identifier)

    status = process_status.status
    if len(status) < 8:
      status = u'{0:s}\t'.format(status)

    sources = u''
    if (process_status.number_of_produced_sources is not None and
        process_status.number_of_produced_sources_delta is not None):
      sources = u'{0:d} ({1:d})'.format(
          process_status.number_of_produced_sources,
          process_status.number_of_produced_sources_delta)

    # This check makes sure the columns are tab aligned.
    if len(sources) < 8:
      sources = u'{0:s}\t'.format(sources)

    events = u''
    if (process_status.number_of_produced_events is not None and
        process_status.number_of_produced_events_delta is not None):
      events = u'{0:d} ({1:d})'.format(
          process_status.number_of_produced_events,
          process_status.number_of_produced_events_delta)

    # This check makes sure the columns are tab aligned.
    if len(events) < 8:
      events = u'{0:s}\t'.format(events)

    used_memory = self._FormatSizeInUnitsOf1024(process_status.used_memory)
    if len(used_memory) < 8:
      used_memory = u'{0:s}\t'.format(used_memory)

    # TODO: shorten display name to fit in 80 chars and show the filename.
    return u'{0:s}\t{1:d}\t{2:s}\t{3:s}\t{4:s}\t{5:s}\t{6:s}'.format(
        identifier, process_status.pid, status, used_memory, sources, events,
        process_status.display_name)

  def _PrintStatusUpdate(self, processing_status):
    """Prints the processing status.

    Args:
      processing_status (ProcessingStatus): processing status.
    """
    if self._stdout_output_writer:
      self._ClearScreen()

    output_text = u'plaso - {0:s} version {1:s}\n\n'.format(
        self.NAME, plaso.__version__)
    self._output_writer.Write(output_text)

    self._PrintStatusHeader()

    # TODO: for win32console get current color and set intensity,
    # write the header separately then reset intensity.
    status_header = (
        u'Identifier\tPID\tStatus\t\tMemory\t\tSources\t\tEvents\t\tFile')
    if not win32console:
      status_header = u'\x1b[1m{0:s}\x1b[0m'.format(status_header)

    status_table = [status_header]

    status_row = self._FormatStatusTableRow(processing_status.foreman_status)
    status_table.append(status_row)

    for worker_status in processing_status.workers_status:
      status_row = self._FormatStatusTableRow(worker_status)
      status_table.append(status_row)

    status_table.append(u'')
    self._output_writer.Write(u'\n'.join(status_table))
    self._output_writer.Write(u'\n')

    if processing_status.aborted:
      self._output_writer.Write(
          u'Processing aborted - waiting for clean up.\n\n')

    # TODO: remove update flicker. For win32console we could set the cursor
    # top left, write the table, clean the remainder of the screen buffer
    # and set the cursor at the end of the table.
    if self._stdout_output_writer:
      # We need to explicitly flush stdout to prevent partial status updates.
      sys.stdout.flush()

  def _PrintStatusUpdateStream(self, processing_status):
    """Prints the processing status as a stream of output.

    Args:
      processing_status (ProcessingStatus): processing status.
    """
    for worker_status in processing_status.workers_status:
      status_line = (
          u'{0:s} (PID: {1:d}) - events produced: {2:d} - file: {3:s} '
          u'- running: {4!s}\n').format(
              worker_status.identifier, worker_status.pid,
              worker_status.number_of_produced_events,
              worker_status.display_name,
              worker_status.status not in definitions.PROCESSING_ERROR_STATUS)
      self._output_writer.Write(status_line)

  def _PrintAnalysisReportsDetails(self, storage):
    """Prints the details of the analysis reports.

    Args:
      storage (BaseStorage): storage writer.
    """
    for index, analysis_report in enumerate(storage.GetAnalysisReports()):
      if index + 1 <= self._number_of_analysis_reports:
        continue

      title = u'Analysis report: {0:d}'.format(index)
      table_view = cli_views.ViewsFactory.GetTableView(
          self._views_format_type, title=title)

      table_view.AddRow([u'String', analysis_report.GetString()])

      table_view.Write(self._output_writer)

  def SetSourcePath(self, path):
    """Set the path of the source to process.

    Args:
      path (str): the path to the source.
    """
    self._source_path = path
